@@ -986,20 +986,12 @@ void FieldTrialList::NotifyFieldTrialGroupSelection(FieldTrial* field_trial) {
     ActivateFieldTrialEntryWhileLocked(field_trial);
   }
 
-  // Recording for stability debugging has to be done inline as a task posted
-  // to an observer may not get executed before a crash.
-  debug::GlobalActivityTracker* tracker = debug::GlobalActivityTracker::Get();
-  if (tracker) {
-    tracker->RecordFieldTrial(field_trial->trial_name(),
-                              field_trial->group_name_internal());
-  }
-
   if (global_->synchronous_observer_) {
     global_->synchronous_observer_->OnFieldTrialGroupFinalized(
         field_trial->trial_name(), field_trial->group_name_internal());
   }
 
-  global_->observer_list_->Notify(
+  global_->observer_list_->NotifySynchronously(
       FROM_HERE, &FieldTrialList::Observer::OnFieldTrialGroupFinalized,
       field_trial->trial_name(), field_trial->group_name_internal());
 }
