@@ -9,22 +9,26 @@
 #include <memory>
 #include <utility>
 
-#include "base/memory/ptr_util.h"
+#include "base/bind.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/api/tabs/tabs_constants.h"
 #include "chrome/browser/extensions/api/tabs/tabs_windows_api.h"
 #include "chrome/browser/extensions/api/tabs/windows_event_router.h"
+#include "chrome/browser/extensions/browser_extension_window_controller.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/recently_audible_helper.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "components/favicon/content/content_favicon_driver.h"
 #include "content/public/browser/favicon_status.h"
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/web_contents.h"
+#include "extensions/common/features/feature.h"
+#include "third_party/blink/public/common/page/page_zoom.h"
 
 using base::DictionaryValue;
 using base::ListValue;
@@ -36,25 +40,29 @@ namespace extensions {
 
 namespace {
 
-namespace tabs = api::tabs;
-
 bool WillDispatchTabUpdatedEvent(
     WebContents* contents,
     const std::set<std::string> changed_property_names,
-    content::BrowserContext* context,
+    content::BrowserContext* browser_context,
+    Feature::Context target_context,
     const Extension* extension,
     Event* event,
     const base::DictionaryValue* listener_filter) {
+  ExtensionTabUtil::ScrubTabBehavior scrub_tab_behavior =
+      ExtensionTabUtil::GetScrubTabBehavior(extension, target_context,
+                                            contents);
   std::unique_ptr<api::tabs::Tab> tab_object =
-      ExtensionTabUtil::CreateTabObject(contents, extension);
+      ExtensionTabUtil::CreateTabObject(contents, scrub_tab_behavior,
+                                        extension);
 
   std::unique_ptr<base::DictionaryValue> tab_value = tab_object->ToValue();
 
-  auto changed_properties = base::MakeUnique<base::DictionaryValue>();
+  auto changed_properties = std::make_unique<base::DictionaryValue>();
   const base::Value* value = nullptr;
   for (const auto& property : changed_property_names) {
     if (tab_value->Get(property, &value))
-      changed_properties->Set(property, base::MakeUnique<base::Value>(*value));
+      changed_properties->Set(property,
+                              std::make_unique<base::Value>(value->Clone()));
   }
 
   event->event_args->Set(1, std::move(changed_properties));
@@ -62,15 +70,38 @@ bool WillDispatchTabUpdatedEvent(
   return true;
 }
 
+bool WillDispatchTabCreatedEvent(WebContents* contents,
+                                 bool active,
+                                 content::BrowserContext* browser_context,
+                                 Feature::Context target_context,
+                                 const Extension* extension,
+                                 Event* event,
+                                 const base::DictionaryValue* listener_filter) {
+  event->event_args->Clear();
+  ExtensionTabUtil::ScrubTabBehavior scrub_tab_behavior =
+      ExtensionTabUtil::GetScrubTabBehavior(extension, target_context,
+                                            contents);
+  std::unique_ptr<base::DictionaryValue> tab_value =
+      ExtensionTabUtil::CreateTabObject(contents, scrub_tab_behavior, extension)
+          ->ToValue();
+  tab_value->SetBoolean(tabs_constants::kSelectedKey, active);
+  tab_value->SetBoolean(tabs_constants::kActiveKey, active);
+  event->event_args->Append(std::move(tab_value));
+  return true;
+}
+
 }  // namespace
 
 TabsEventRouter::TabEntry::TabEntry(TabsEventRouter* router,
                                     content::WebContents* contents)
     : WebContentsObserver(contents),
       complete_waiting_on_load_(false),
-      was_audible_(contents->WasRecentlyAudible()),
+      was_audible_(false),
       was_muted_(contents->IsAudioMuted()),
-      router_(router) {}
+      router_(router) {
+  auto* audible_helper = RecentlyAudibleHelper::FromWebContents(contents);
+  was_audible_ = audible_helper->WasRecentlyAudible();
+}
 
 std::set<std::string> TabsEventRouter::TabEntry::UpdateLoadState() {
   // The tab may go in & out of loading (for instance if iframes navigate).
@@ -116,8 +147,7 @@ void TabsEventRouter::TabEntry::NavigationEntryCommitted(
   router_->TabUpdated(this, std::move(changed_property_names));
 }
 
-void TabsEventRouter::TabEntry::TitleWasSet(content::NavigationEntry* entry,
-                                            bool explicit_set) {
+void TabsEventRouter::TabEntry::TitleWasSet(content::NavigationEntry* entry) {
   std::set<std::string> changed_property_names;
   changed_property_names.insert(tabs_constants::kTitleKey);
   router_->TabUpdated(this, std::move(changed_property_names));
@@ -133,18 +163,17 @@ void TabsEventRouter::TabEntry::WebContentsDestroyed() {
 }
 
 TabsEventRouter::TabsEventRouter(Profile* profile)
-    : profile_(profile),
-      favicon_scoped_observer_(this),
-      browser_tab_strip_tracker_(this, this, this),
-      tab_manager_scoped_observer_(this) {
+    : profile_(profile), browser_tab_strip_tracker_(this, this) {
   DCHECK(!profile->IsOffTheRecord());
 
+  BrowserList::AddObserver(this);
   browser_tab_strip_tracker_.Init();
 
   tab_manager_scoped_observer_.Add(g_browser_process->GetTabManager());
 }
 
 TabsEventRouter::~TabsEventRouter() {
+  BrowserList::RemoveObserver(this);
 }
 
 bool TabsEventRouter::ShouldTrackBrowser(Browser* browser) {
@@ -152,28 +181,6 @@ bool TabsEventRouter::ShouldTrackBrowser(Browser* browser) {
          ExtensionTabUtil::BrowserSupportsTabs(browser);
 }
 
-void TabsEventRouter::RegisterForTabNotifications(WebContents* contents) {
-  favicon_scoped_observer_.Add(
-      favicon::ContentFaviconDriver::FromWebContents(contents));
-
-  ZoomController::FromWebContents(contents)->AddObserver(this);
-
-  int tab_id = ExtensionTabUtil::GetTabId(contents);
-  DCHECK(tab_entries_.find(tab_id) == tab_entries_.end());
-  tab_entries_[tab_id] = base::MakeUnique<TabEntry>(this, contents);
-}
-
-void TabsEventRouter::UnregisterForTabNotifications(WebContents* contents) {
-  favicon_scoped_observer_.Remove(
-      favicon::ContentFaviconDriver::FromWebContents(contents));
-
-  ZoomController::FromWebContents(contents)->RemoveObserver(this);
-
-  int tab_id = ExtensionTabUtil::GetTabId(contents);
-  int removed_count = tab_entries_.erase(tab_id);
-  DCHECK_GT(removed_count, 0);
-}
-
 void TabsEventRouter::OnBrowserSetLastActive(Browser* browser) {
   TabsWindowsAPI* tabs_window_api = TabsWindowsAPI::Get(profile_);
   if (tabs_window_api) {
@@ -182,42 +189,142 @@ void TabsEventRouter::OnBrowserSetLastActive(Browser* browser) {
   }
 }
 
-static bool WillDispatchTabCreatedEvent(
-    WebContents* contents,
-    bool active,
-    content::BrowserContext* context,
-    const Extension* extension,
-    Event* event,
-    const base::DictionaryValue* listener_filter) {
-  event->event_args->Clear();
-  std::unique_ptr<base::DictionaryValue> tab_value =
-      ExtensionTabUtil::CreateTabObject(contents, extension)->ToValue();
-  tab_value->SetBoolean(tabs_constants::kSelectedKey, active);
-  tab_value->SetBoolean(tabs_constants::kActiveKey, active);
-  event->event_args->Append(std::move(tab_value));
-  return true;
+void TabsEventRouter::OnTabStripModelChanged(
+    TabStripModel* tab_strip_model,
+    const TabStripModelChange& change,
+    const TabStripSelectionChange& selection) {
+  switch (change.type()) {
+    case TabStripModelChange::kInserted: {
+      for (const auto& contents : change.GetInsert()->contents) {
+        DispatchTabInsertedAt(tab_strip_model, contents.contents,
+                              contents.index,
+                              selection.new_contents == contents.contents);
+      }
+      break;
+    }
+    case TabStripModelChange::kRemoved: {
+      const bool will_be_deleted = change.GetRemove()->will_be_deleted;
+      for (const auto& contents : change.GetRemove()->contents) {
+        if (will_be_deleted) {
+          DispatchTabClosingAt(tab_strip_model, contents.contents,
+                               contents.index);
+        }
+
+        DispatchTabDetachedAt(contents.contents, contents.index,
+                              selection.old_contents == contents.contents);
+      }
+      break;
+    }
+    case TabStripModelChange::kMoved: {
+      auto* move = change.GetMove();
+      DispatchTabMoved(move->contents, move->from_index, move->to_index);
+      break;
+    }
+    case TabStripModelChange::kReplaced: {
+      auto* replace = change.GetReplace();
+      DispatchTabReplacedAt(replace->old_contents, replace->new_contents,
+                            replace->index);
+      break;
+    }
+    case TabStripModelChange::kSelectionOnly:
+      break;
+  }
+
+  if (tab_strip_model->empty())
+    return;
+
+  if (selection.active_tab_changed()) {
+    DispatchActiveTabChanged(selection.old_contents, selection.new_contents,
+                             selection.new_model.active());
+  }
+
+  if (selection.selection_changed()) {
+    DispatchTabSelectionChanged(tab_strip_model, selection.old_model);
+  }
 }
 
-void TabsEventRouter::TabCreatedAt(WebContents* contents,
+void TabsEventRouter::TabChangedAt(WebContents* contents,
                                    int index,
-                                   bool active) {
-  Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  auto event = base::MakeUnique<Event>(events::TABS_ON_CREATED,
-                                       tabs::OnCreated::kEventName,
-                                       std::move(args), profile);
-  event->user_gesture = EventRouter::USER_GESTURE_NOT_ENABLED;
-  event->will_dispatch_callback =
-      base::Bind(&WillDispatchTabCreatedEvent, contents, active);
-  EventRouter::Get(profile)->BroadcastEvent(std::move(event));
-
-  RegisterForTabNotifications(contents);
+                                   TabChangeType change_type) {
+  TabEntry* entry = GetTabEntry(contents);
+  // TabClosingAt() may have already removed the entry for |contents| even
+  // though the tab has not yet been detached.
+  if (entry)
+    TabUpdated(entry, entry->UpdateLoadState());
 }
 
-void TabsEventRouter::TabInsertedAt(TabStripModel* tab_strip_model,
-                                    WebContents* contents,
-                                    int index,
-                                    bool active) {
+void TabsEventRouter::TabPinnedStateChanged(TabStripModel* tab_strip_model,
+                                            WebContents* contents,
+                                            int index) {
+  std::set<std::string> changed_property_names;
+  changed_property_names.insert(tabs_constants::kPinnedKey);
+  DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
+}
+
+void TabsEventRouter::OnZoomChanged(
+    const ZoomController::ZoomChangedEventData& data) {
+  DCHECK(data.web_contents);
+  int tab_id = ExtensionTabUtil::GetTabId(data.web_contents);
+  if (tab_id < 0)
+    return;
+
+  // Prepare the zoom change information.
+  api::tabs::OnZoomChange::ZoomChangeInfo zoom_change_info;
+  zoom_change_info.tab_id = tab_id;
+  zoom_change_info.old_zoom_factor =
+      blink::PageZoomLevelToZoomFactor(data.old_zoom_level);
+  zoom_change_info.new_zoom_factor =
+      blink::PageZoomLevelToZoomFactor(data.new_zoom_level);
+  ZoomModeToZoomSettings(data.zoom_mode, &zoom_change_info.zoom_settings);
+
+  // Dispatch the |onZoomChange| event.
+  Profile* profile =
+      Profile::FromBrowserContext(data.web_contents->GetBrowserContext());
+  DispatchEvent(profile, events::TABS_ON_ZOOM_CHANGE,
+                api::tabs::OnZoomChange::kEventName,
+                api::tabs::OnZoomChange::Create(zoom_change_info),
+                EventRouter::USER_GESTURE_UNKNOWN);
+}
+
+void TabsEventRouter::OnFaviconUpdated(
+    favicon::FaviconDriver* favicon_driver,
+    NotificationIconType notification_icon_type,
+    const GURL& icon_url,
+    bool icon_url_changed,
+    const gfx::Image& image) {
+  if (notification_icon_type == NON_TOUCH_16_DIP && icon_url_changed) {
+    favicon::ContentFaviconDriver* content_favicon_driver =
+        static_cast<favicon::ContentFaviconDriver*>(favicon_driver);
+    FaviconUrlUpdated(content_favicon_driver->web_contents());
+  }
+}
+
+void TabsEventRouter::OnDiscardedStateChange(
+    WebContents* contents,
+    ::mojom::LifecycleUnitDiscardReason reason,
+    bool is_discarded) {
+  std::set<std::string> changed_property_names;
+  // If the "discarded" property changes, so does the "status" property:
+  // - a discarded tab has status "unloaded", and will transition to "loading"
+  //   on un-discarding; and,
+  // - a tab can only be discarded if its status is "complete" or "loading", in
+  //   which case it will transition to "unloaded".
+  changed_property_names.insert(tabs_constants::kDiscardedKey);
+  changed_property_names.insert(tabs_constants::kStatusKey);
+  DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
+}
+
+void TabsEventRouter::OnAutoDiscardableStateChange(WebContents* contents,
+                                                   bool is_auto_discardable) {
+  std::set<std::string> changed_property_names;
+  changed_property_names.insert(tabs_constants::kAutoDiscardableKey);
+  DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
+}
+
+void TabsEventRouter::DispatchTabInsertedAt(TabStripModel* tab_strip_model,
+                                            WebContents* contents,
+                                            int index,
+                                            bool active) {
   if (!GetTabEntry(contents)) {
     // We've never seen this tab, send create event as long as we're not in the
     // constructor.
@@ -236,42 +343,20 @@ void TabsEventRouter::TabInsertedAt(TabStripModel* tab_strip_model,
       new base::DictionaryValue());
   object_args->Set(
       tabs_constants::kNewWindowIdKey,
-      base::MakeUnique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
+      std::make_unique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
   object_args->Set(tabs_constants::kNewPositionKey,
-                   base::MakeUnique<Value>(index));
+                   std::make_unique<Value>(index));
   args->Append(std::move(object_args));
 
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
-  DispatchEvent(profile, events::TABS_ON_ATTACHED, tabs::OnAttached::kEventName,
-                std::move(args), EventRouter::USER_GESTURE_UNKNOWN);
+  DispatchEvent(profile, events::TABS_ON_ATTACHED,
+                api::tabs::OnAttached::kEventName, std::move(args),
+                EventRouter::USER_GESTURE_UNKNOWN);
 }
 
-void TabsEventRouter::TabDetachedAt(WebContents* contents, int index) {
-  if (!GetTabEntry(contents)) {
-    // The tab was removed. Don't send detach event.
-    return;
-  }
-
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  args->AppendInteger(ExtensionTabUtil::GetTabId(contents));
-
-  std::unique_ptr<base::DictionaryValue> object_args(
-      new base::DictionaryValue());
-  object_args->Set(
-      tabs_constants::kOldWindowIdKey,
-      base::MakeUnique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
-  object_args->Set(tabs_constants::kOldPositionKey,
-                   base::MakeUnique<Value>(index));
-  args->Append(std::move(object_args));
-
-  Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
-  DispatchEvent(profile, events::TABS_ON_DETACHED, tabs::OnDetached::kEventName,
-                std::move(args), EventRouter::USER_GESTURE_UNKNOWN);
-}
-
-void TabsEventRouter::TabClosingAt(TabStripModel* tab_strip_model,
-                                   WebContents* contents,
-                                   int index) {
+void TabsEventRouter::DispatchTabClosingAt(TabStripModel* tab_strip_model,
+                                           WebContents* contents,
+                                           int index) {
   int tab_id = ExtensionTabUtil::GetTabId(contents);
 
   std::unique_ptr<base::ListValue> args(new base::ListValue);
@@ -286,23 +371,49 @@ void TabsEventRouter::TabClosingAt(TabStripModel* tab_strip_model,
   args->Append(std::move(object_args));
 
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
-  DispatchEvent(profile, events::TABS_ON_REMOVED, tabs::OnRemoved::kEventName,
-                std::move(args), EventRouter::USER_GESTURE_UNKNOWN);
+  DispatchEvent(profile, events::TABS_ON_REMOVED,
+                api::tabs::OnRemoved::kEventName, std::move(args),
+                EventRouter::USER_GESTURE_UNKNOWN);
 
   UnregisterForTabNotifications(contents);
 }
 
-void TabsEventRouter::ActiveTabChanged(WebContents* old_contents,
-                                       WebContents* new_contents,
-                                       int index,
-                                       int reason) {
-  auto args = base::MakeUnique<base::ListValue>();
+void TabsEventRouter::DispatchTabDetachedAt(WebContents* contents,
+                                            int index,
+                                            bool was_active) {
+  if (!GetTabEntry(contents)) {
+    // The tab was removed. Don't send detach event.
+    return;
+  }
+
+  std::unique_ptr<base::ListValue> args(new base::ListValue);
+  args->AppendInteger(ExtensionTabUtil::GetTabId(contents));
+
+  std::unique_ptr<base::DictionaryValue> object_args(
+      new base::DictionaryValue());
+  object_args->Set(
+      tabs_constants::kOldWindowIdKey,
+      std::make_unique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
+  object_args->Set(tabs_constants::kOldPositionKey,
+                   std::make_unique<Value>(index));
+  args->Append(std::move(object_args));
+
+  Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
+  DispatchEvent(profile, events::TABS_ON_DETACHED,
+                api::tabs::OnDetached::kEventName, std::move(args),
+                EventRouter::USER_GESTURE_UNKNOWN);
+}
+
+void TabsEventRouter::DispatchActiveTabChanged(WebContents* old_contents,
+                                               WebContents* new_contents,
+                                               int index) {
+  auto args = std::make_unique<base::ListValue>();
   int tab_id = ExtensionTabUtil::GetTabId(new_contents);
   args->AppendInteger(tab_id);
 
-  auto object_args = base::MakeUnique<base::DictionaryValue>();
+  auto object_args = std::make_unique<base::DictionaryValue>();
   object_args->Set(tabs_constants::kWindowIdKey,
-                   base::MakeUnique<Value>(
+                   std::make_unique<Value>(
                        ExtensionTabUtil::GetWindowIdOfTab(new_contents)));
   args->Append(object_args->CreateDeepCopy());
 
@@ -310,26 +421,24 @@ void TabsEventRouter::ActiveTabChanged(WebContents* old_contents,
   // deprecated events take two arguments: tabId, {windowId}.
   Profile* profile =
       Profile::FromBrowserContext(new_contents->GetBrowserContext());
-  EventRouter::UserGestureState gesture =
-      reason & CHANGE_REASON_USER_GESTURE
-      ? EventRouter::USER_GESTURE_ENABLED
-      : EventRouter::USER_GESTURE_NOT_ENABLED;
+
   DispatchEvent(profile, events::TABS_ON_SELECTION_CHANGED,
-                tabs::OnSelectionChanged::kEventName, args->CreateDeepCopy(),
-                gesture);
+                api::tabs::OnSelectionChanged::kEventName,
+                args->CreateDeepCopy(), EventRouter::USER_GESTURE_UNKNOWN);
   DispatchEvent(profile, events::TABS_ON_ACTIVE_CHANGED,
-                tabs::OnActiveChanged::kEventName, std::move(args), gesture);
+                api::tabs::OnActiveChanged::kEventName, std::move(args),
+                EventRouter::USER_GESTURE_UNKNOWN);
 
   // The onActivated event takes one argument: {windowId, tabId}.
-  auto on_activated_args = base::MakeUnique<base::ListValue>();
-  object_args->Set(tabs_constants::kTabIdKey, base::MakeUnique<Value>(tab_id));
+  auto on_activated_args = std::make_unique<base::ListValue>();
+  object_args->Set(tabs_constants::kTabIdKey, std::make_unique<Value>(tab_id));
   on_activated_args->Append(std::move(object_args));
-  DispatchEvent(profile, events::TABS_ON_ACTIVATED,
-                tabs::OnActivated::kEventName, std::move(on_activated_args),
-                gesture);
+  DispatchEvent(
+      profile, events::TABS_ON_ACTIVATED, api::tabs::OnActivated::kEventName,
+      std::move(on_activated_args), EventRouter::USER_GESTURE_UNKNOWN);
 }
 
-void TabsEventRouter::TabSelectionChanged(
+void TabsEventRouter::DispatchTabSelectionChanged(
     TabStripModel* tab_strip_model,
     const ui::ListSelectionModel& old_model) {
   ui::ListSelectionModel::SelectedIndices new_selection =
@@ -350,7 +459,7 @@ void TabsEventRouter::TabSelectionChanged(
 
   select_info->Set(
       tabs_constants::kWindowIdKey,
-      base::MakeUnique<Value>(
+      std::make_unique<Value>(
           ExtensionTabUtil::GetWindowIdOfTabStripModel(tab_strip_model)));
 
   select_info->Set(tabs_constants::kTabIdsKey, std::move(all_tabs));
@@ -359,17 +468,17 @@ void TabsEventRouter::TabSelectionChanged(
   // The onHighlighted event replaced onHighlightChanged.
   Profile* profile = tab_strip_model->profile();
   DispatchEvent(profile, events::TABS_ON_HIGHLIGHT_CHANGED,
-                tabs::OnHighlightChanged::kEventName,
+                api::tabs::OnHighlightChanged::kEventName,
                 std::unique_ptr<base::ListValue>(args->DeepCopy()),
                 EventRouter::USER_GESTURE_UNKNOWN);
   DispatchEvent(profile, events::TABS_ON_HIGHLIGHTED,
-                tabs::OnHighlighted::kEventName, std::move(args),
+                api::tabs::OnHighlighted::kEventName, std::move(args),
                 EventRouter::USER_GESTURE_UNKNOWN);
 }
 
-void TabsEventRouter::TabMoved(WebContents* contents,
-                               int from_index,
-                               int to_index) {
+void TabsEventRouter::DispatchTabMoved(WebContents* contents,
+                                       int from_index,
+                                       int to_index) {
   std::unique_ptr<base::ListValue> args(new base::ListValue);
   args->AppendInteger(ExtensionTabUtil::GetTabId(contents));
 
@@ -377,21 +486,60 @@ void TabsEventRouter::TabMoved(WebContents* contents,
       new base::DictionaryValue());
   object_args->Set(
       tabs_constants::kWindowIdKey,
-      base::MakeUnique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
+      std::make_unique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
   object_args->Set(tabs_constants::kFromIndexKey,
-                   base::MakeUnique<Value>(from_index));
+                   std::make_unique<Value>(from_index));
   object_args->Set(tabs_constants::kToIndexKey,
-                   base::MakeUnique<Value>(to_index));
+                   std::make_unique<Value>(to_index));
   args->Append(std::move(object_args));
 
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
-  DispatchEvent(profile, events::TABS_ON_MOVED, tabs::OnMoved::kEventName,
+  DispatchEvent(profile, events::TABS_ON_MOVED, api::tabs::OnMoved::kEventName,
                 std::move(args), EventRouter::USER_GESTURE_UNKNOWN);
 }
 
+void TabsEventRouter::DispatchTabReplacedAt(WebContents* old_contents,
+                                            WebContents* new_contents,
+                                            int index) {
+  // Notify listeners that the next tabs closing or being added are due to
+  // WebContents being swapped.
+  const int new_tab_id = ExtensionTabUtil::GetTabId(new_contents);
+  const int old_tab_id = ExtensionTabUtil::GetTabId(old_contents);
+  std::unique_ptr<base::ListValue> args(new base::ListValue);
+  args->AppendInteger(new_tab_id);
+  args->AppendInteger(old_tab_id);
+
+  DispatchEvent(Profile::FromBrowserContext(new_contents->GetBrowserContext()),
+                events::TABS_ON_REPLACED, api::tabs::OnReplaced::kEventName,
+                std::move(args), EventRouter::USER_GESTURE_UNKNOWN);
+
+  UnregisterForTabNotifications(old_contents);
+
+  if (!GetTabEntry(new_contents))
+    RegisterForTabNotifications(new_contents);
+}
+
+void TabsEventRouter::TabCreatedAt(WebContents* contents,
+                                   int index,
+                                   bool active) {
+  Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
+  std::unique_ptr<base::ListValue> args(new base::ListValue);
+  auto event = std::make_unique<Event>(events::TABS_ON_CREATED,
+                                       api::tabs::OnCreated::kEventName,
+                                       std::move(args), profile);
+  event->user_gesture = EventRouter::USER_GESTURE_NOT_ENABLED;
+  event->will_dispatch_callback =
+      base::BindRepeating(&WillDispatchTabCreatedEvent, contents, active);
+  EventRouter::Get(profile)->BroadcastEvent(std::move(event));
+
+  RegisterForTabNotifications(contents);
+}
+
 void TabsEventRouter::TabUpdated(TabEntry* entry,
                                  std::set<std::string> changed_property_names) {
-  bool audible = entry->web_contents()->WasRecentlyAudible();
+  auto* audible_helper =
+      RecentlyAudibleHelper::FromWebContents(entry->web_contents());
+  bool audible = audible_helper->WasRecentlyAudible();
   if (entry->SetAudible(audible)) {
     changed_property_names.insert(tabs_constants::kAudibleKey);
   }
@@ -426,7 +574,7 @@ void TabsEventRouter::DispatchEvent(
   if (!profile_->IsSameProfile(profile) || !event_router)
     return;
 
-  auto event = base::MakeUnique<Event>(histogram_value, event_name,
+  auto event = std::make_unique<Event>(histogram_value, event_name,
                                        std::move(args), profile);
   event->user_gesture = user_gesture;
   event_router->BroadcastEvent(std::move(event));
@@ -453,113 +601,42 @@ void TabsEventRouter::DispatchTabUpdatedEvent(
   // WillDispatchTabUpdatedEvent.
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
 
-  auto event = base::MakeUnique<Event>(events::TABS_ON_UPDATED,
-                                       tabs::OnUpdated::kEventName,
+  auto event = std::make_unique<Event>(events::TABS_ON_UPDATED,
+                                       api::tabs::OnUpdated::kEventName,
                                        std::move(args_base), profile);
   event->user_gesture = EventRouter::USER_GESTURE_NOT_ENABLED;
   event->will_dispatch_callback =
-      base::Bind(&WillDispatchTabUpdatedEvent, contents,
-                 std::move(changed_property_names));
+      base::BindRepeating(&WillDispatchTabUpdatedEvent, contents,
+                          std::move(changed_property_names));
   EventRouter::Get(profile)->BroadcastEvent(std::move(event));
 }
 
+void TabsEventRouter::RegisterForTabNotifications(WebContents* contents) {
+  favicon_scoped_observer_.Add(
+      favicon::ContentFaviconDriver::FromWebContents(contents));
+
+  ZoomController::FromWebContents(contents)->AddObserver(this);
+
+  int tab_id = ExtensionTabUtil::GetTabId(contents);
+  DCHECK(tab_entries_.find(tab_id) == tab_entries_.end());
+  tab_entries_[tab_id] = std::make_unique<TabEntry>(this, contents);
+}
+
+void TabsEventRouter::UnregisterForTabNotifications(WebContents* contents) {
+  favicon_scoped_observer_.Remove(
+      favicon::ContentFaviconDriver::FromWebContents(contents));
+
+  ZoomController::FromWebContents(contents)->RemoveObserver(this);
+
+  int tab_id = ExtensionTabUtil::GetTabId(contents);
+  int removed_count = tab_entries_.erase(tab_id);
+  DCHECK_GT(removed_count, 0);
+}
+
 TabsEventRouter::TabEntry* TabsEventRouter::GetTabEntry(WebContents* contents) {
   const auto it = tab_entries_.find(ExtensionTabUtil::GetTabId(contents));
 
   return it == tab_entries_.end() ? nullptr : it->second.get();
 }
 
-void TabsEventRouter::TabChangedAt(WebContents* contents,
-                                   int index,
-                                   TabChangeType change_type) {
-  TabEntry* entry = GetTabEntry(contents);
-  // TabClosingAt() may have already removed the entry for |contents| even
-  // though the tab has not yet been detached.
-  if (entry)
-    TabUpdated(entry, entry->UpdateLoadState());
-}
-
-void TabsEventRouter::TabReplacedAt(TabStripModel* tab_strip_model,
-                                    WebContents* old_contents,
-                                    WebContents* new_contents,
-                                    int index) {
-  // Notify listeners that the next tabs closing or being added are due to
-  // WebContents being swapped.
-  const int new_tab_id = ExtensionTabUtil::GetTabId(new_contents);
-  const int old_tab_id = ExtensionTabUtil::GetTabId(old_contents);
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  args->AppendInteger(new_tab_id);
-  args->AppendInteger(old_tab_id);
-
-  DispatchEvent(Profile::FromBrowserContext(new_contents->GetBrowserContext()),
-                events::TABS_ON_REPLACED, tabs::OnReplaced::kEventName,
-                std::move(args), EventRouter::USER_GESTURE_UNKNOWN);
-
-  UnregisterForTabNotifications(old_contents);
-
-  if (!GetTabEntry(new_contents))
-    RegisterForTabNotifications(new_contents);
-}
-
-void TabsEventRouter::TabPinnedStateChanged(TabStripModel* tab_strip_model,
-                                            WebContents* contents,
-                                            int index) {
-  std::set<std::string> changed_property_names;
-  changed_property_names.insert(tabs_constants::kPinnedKey);
-  DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
-}
-
-void TabsEventRouter::OnZoomChanged(
-    const ZoomController::ZoomChangedEventData& data) {
-  DCHECK(data.web_contents);
-  int tab_id = ExtensionTabUtil::GetTabId(data.web_contents);
-  if (tab_id < 0)
-    return;
-
-  // Prepare the zoom change information.
-  api::tabs::OnZoomChange::ZoomChangeInfo zoom_change_info;
-  zoom_change_info.tab_id = tab_id;
-  zoom_change_info.old_zoom_factor =
-      content::ZoomLevelToZoomFactor(data.old_zoom_level);
-  zoom_change_info.new_zoom_factor =
-      content::ZoomLevelToZoomFactor(data.new_zoom_level);
-  ZoomModeToZoomSettings(data.zoom_mode,
-                         &zoom_change_info.zoom_settings);
-
-  // Dispatch the |onZoomChange| event.
-  Profile* profile = Profile::FromBrowserContext(
-      data.web_contents->GetBrowserContext());
-  DispatchEvent(profile, events::TABS_ON_ZOOM_CHANGE,
-                tabs::OnZoomChange::kEventName,
-                api::tabs::OnZoomChange::Create(zoom_change_info),
-                EventRouter::USER_GESTURE_UNKNOWN);
-}
-
-void TabsEventRouter::OnFaviconUpdated(
-    favicon::FaviconDriver* favicon_driver,
-    NotificationIconType notification_icon_type,
-    const GURL& icon_url,
-    bool icon_url_changed,
-    const gfx::Image& image) {
-  if (notification_icon_type == NON_TOUCH_16_DIP && icon_url_changed) {
-    favicon::ContentFaviconDriver* content_favicon_driver =
-        static_cast<favicon::ContentFaviconDriver*>(favicon_driver);
-    FaviconUrlUpdated(content_favicon_driver->web_contents());
-  }
-}
-
-void TabsEventRouter::OnDiscardedStateChange(WebContents* contents,
-                                             bool is_discarded) {
-  std::set<std::string> changed_property_names;
-  changed_property_names.insert(tabs_constants::kDiscardedKey);
-  DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
-}
-
-void TabsEventRouter::OnAutoDiscardableStateChange(WebContents* contents,
-                                                   bool is_auto_discardable) {
-  std::set<std::string> changed_property_names;
-  changed_property_names.insert(tabs_constants::kAutoDiscardableKey);
-  DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
-}
-
 }  // namespace extensions
