@@ -12,25 +12,32 @@
 
 #include "base/bind.h"
 #include "base/guid.h"
-#include "base/memory/ptr_util.h"
+#include "base/json/json_writer.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
+#include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/android/offline_pages/prefetch/prefetch_background_task.h"
+#include "build/build_config.h"
 #include "chrome/browser/offline_pages/offline_page_model_factory.h"
 #include "chrome/browser/offline_pages/prefetch/prefetch_service_factory.h"
+#include "chrome/browser/offline_pages/prefetch/prefetched_pages_notifier.h"
 #include "chrome/browser/offline_pages/request_coordinator_factory.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/profiles/profile_key.h"
 #include "chrome/common/channel_info.h"
 #include "chrome/common/chrome_content_client.h"
 #include "components/offline_pages/core/client_namespace_constants.h"
 #include "components/offline_pages/core/offline_page_feature.h"
 #include "components/offline_pages/core/prefetch/generate_page_bundle_request.h"
 #include "components/offline_pages/core/prefetch/get_operation_request.h"
+#include "components/offline_pages/core/prefetch/prefetch_background_task_handler.h"
+#include "components/offline_pages/core/prefetch/prefetch_dispatcher.h"
 #include "components/offline_pages/core/prefetch/prefetch_downloader.h"
+#include "components/offline_pages/core/prefetch/prefetch_prefs.h"
 #include "components/offline_pages/core/prefetch/prefetch_service.h"
+#include "components/offline_pages/core/prefetch/prefetch_types.h"
 #include "content/public/browser/web_ui.h"
 #include "net/base/network_change_notifier.h"
 
@@ -49,10 +56,8 @@ std::string GetStringFromDeletePageResult(
       return "Store failure";
     case offline_pages::DeletePageResult::DEVICE_FAILURE:
       return "Device failure";
-    case offline_pages::DeletePageResult::NOT_FOUND:
+    case offline_pages::DeletePageResult::DEPRECATED_NOT_FOUND:
       return "Not found";
-    case offline_pages::DeletePageResult::RESULT_COUNT:
-      break;
   }
   NOTREACHED();
   return "Unknown";
@@ -73,55 +78,12 @@ std::string GetStringFromSavePageStatus() {
   return "Available";
 }
 
-std::string GetStringFromPrefetchRequestStatus(
-    offline_pages::PrefetchRequestStatus status) {
-  switch (status) {
-    case offline_pages::PrefetchRequestStatus::SUCCESS:
-      return "Success";
-    case offline_pages::PrefetchRequestStatus::SHOULD_RETRY_WITHOUT_BACKOFF:
-      return "Retry w/out backoff";
-    case offline_pages::PrefetchRequestStatus::SHOULD_RETRY_WITH_BACKOFF:
-      return "Retry w/ backoff";
-    case offline_pages::PrefetchRequestStatus::SHOULD_SUSPEND:
-      return "Suspend";
-    default:
-      NOTREACHED();
-      return "Unknown";
-  }
-}
-
-std::string GetStringRenderPageInfoList(
-    const std::vector<offline_pages::RenderPageInfo>& pages) {
-  std::string str("[\n");
-  bool first = true;
-  for (const auto& page : pages) {
-    if (first)
-      first = false;
-    else
-      str += ",\n";
-    str += base::StringPrintf(
-        "  {\n"
-        "    url: \"%s\",\n"
-        "    redirect_url: \"%s\",\n"
-        "    status: %d,\n"
-        "    body_name: \"%s\",\n"
-        "    body_length: %lld\n"
-        "  }",
-        page.url.c_str(), page.redirect_url.c_str(),
-        static_cast<int>(page.status), page.body_name.c_str(),
-        static_cast<long long>(page.body_length));
-  }
-  str += "\n]";
-  return str;
-}
-
 }  // namespace
 
 OfflineInternalsUIMessageHandler::OfflineInternalsUIMessageHandler()
     : offline_page_model_(nullptr),
       request_coordinator_(nullptr),
-      prefetch_service_(nullptr),
-      weak_ptr_factory_(this) {}
+      prefetch_service_(nullptr) {}
 
 OfflineInternalsUIMessageHandler::~OfflineInternalsUIMessageHandler() {}
 
@@ -130,10 +92,10 @@ void OfflineInternalsUIMessageHandler::HandleDeleteSelectedPages(
   std::string callback_id;
   CHECK(args->GetString(0, &callback_id));
 
-  std::vector<int64_t> offline_ids;
   const base::ListValue* offline_ids_from_arg;
   args->GetList(1, &offline_ids_from_arg);
 
+  std::vector<int64_t> offline_ids;
   for (size_t i = 0; i < offline_ids_from_arg->GetSize(); i++) {
     std::string value;
     offline_ids_from_arg->GetString(i, &value);
@@ -142,8 +104,10 @@ void OfflineInternalsUIMessageHandler::HandleDeleteSelectedPages(
     offline_ids.push_back(int_value);
   }
 
-  offline_page_model_->DeletePagesByOfflineId(
-      offline_ids,
+  offline_pages::PageCriteria criteria;
+  criteria.offline_ids = std::move(offline_ids);
+  offline_page_model_->DeletePagesWithCriteria(
+      criteria,
       base::Bind(&OfflineInternalsUIMessageHandler::HandleDeletedPagesCallback,
                  weak_ptr_factory_.GetWeakPtr(), callback_id));
 }
@@ -193,45 +157,50 @@ void OfflineInternalsUIMessageHandler::HandleDeletedRequestsCallback(
 void OfflineInternalsUIMessageHandler::HandleStoredPagesCallback(
     std::string callback_id,
     const offline_pages::MultipleOfflinePageItemResult& pages) {
-  base::ListValue results;
-
+  std::vector<base::Value> results;
   for (const auto& page : pages) {
-    auto offline_page = base::MakeUnique<base::DictionaryValue>();
-    offline_page->SetString("onlineUrl", page.url.spec());
-    offline_page->SetString("namespace", page.client_id.name_space);
-    offline_page->SetDouble("size", page.file_size);
-    offline_page->SetString("id", std::to_string(page.offline_id));
-    offline_page->SetString("filePath", page.file_path.MaybeAsASCII());
-    offline_page->SetDouble("creationTime", page.creation_time.ToJsTime());
-    offline_page->SetDouble("lastAccessTime", page.last_access_time.ToJsTime());
-    offline_page->SetInteger("accessCount", page.access_count);
-    offline_page->SetString("originalUrl", page.original_url.spec());
-    results.Append(std::move(offline_page));
+    base::Value offline_page(base::Value::Type::DICTIONARY);
+    offline_page.SetStringKey("onlineUrl", page.url.spec());
+    offline_page.SetStringKey("namespace", page.client_id.name_space);
+    offline_page.SetDoubleKey("size", page.file_size);
+    offline_page.SetStringKey("id", std::to_string(page.offline_id));
+    offline_page.SetStringKey("filePath", page.file_path.MaybeAsASCII());
+    offline_page.SetDoubleKey("creationTime", page.creation_time.ToJsTime());
+    offline_page.SetDoubleKey("lastAccessTime",
+                              page.last_access_time.ToJsTime());
+    offline_page.SetIntKey("accessCount", page.access_count);
+    offline_page.SetStringKey("originalUrl",
+                              page.original_url_if_different.spec());
+    offline_page.SetStringKey("requestOrigin", page.request_origin);
+    results.push_back(std::move(offline_page));
   }
-  ResolveJavascriptCallback(base::Value(callback_id), results);
+  // Sort by creation order.
+  std::sort(results.begin(), results.end(), [](const auto& a, const auto& b) {
+    return a.FindKey({"creationTime"})->GetDouble() <
+           b.FindKey({"creationTime"})->GetDouble();
+  });
+
+  ResolveJavascriptCallback(base::Value(callback_id),
+                            base::Value(std::move(results)));
 }
 
 void OfflineInternalsUIMessageHandler::HandleRequestQueueCallback(
     std::string callback_id,
-    offline_pages::GetRequestsResult result,
     std::vector<std::unique_ptr<offline_pages::SavePageRequest>> requests) {
   base::ListValue save_page_requests;
-  if (result == offline_pages::GetRequestsResult::SUCCESS) {
-    for (const auto& request : requests) {
-      auto save_page_request = base::MakeUnique<base::DictionaryValue>();
-      save_page_request->SetString("onlineUrl", request->url().spec());
-      save_page_request->SetDouble("creationTime",
-                                   request->creation_time().ToJsTime());
-      save_page_request->SetString("status", GetStringFromSavePageStatus());
-      save_page_request->SetString("namespace",
-                                   request->client_id().name_space);
-      save_page_request->SetDouble("lastAttempt",
-                                   request->last_attempt_time().ToJsTime());
-      save_page_request->SetString("id", std::to_string(request->request_id()));
-      save_page_request->SetString("originalUrl",
-                                   request->original_url().spec());
-      save_page_requests.Append(std::move(save_page_request));
-    }
+  for (const auto& request : requests) {
+    auto save_page_request = std::make_unique<base::DictionaryValue>();
+    save_page_request->SetString("onlineUrl", request->url().spec());
+    save_page_request->SetDouble("creationTime",
+                                 request->creation_time().ToJsTime());
+    save_page_request->SetString("status", GetStringFromSavePageStatus());
+    save_page_request->SetString("namespace", request->client_id().name_space);
+    save_page_request->SetDouble("lastAttemptTime",
+                                 request->last_attempt_time().ToJsTime());
+    save_page_request->SetString("id", std::to_string(request->request_id()));
+    save_page_request->SetString("originalUrl", request->original_url().spec());
+    save_page_request->SetString("requestOrigin", request->request_origin());
+    save_page_requests.Append(std::move(save_page_request));
   }
   ResolveJavascriptCallback(base::Value(callback_id), save_page_requests);
 }
@@ -243,7 +212,7 @@ void OfflineInternalsUIMessageHandler::HandleGetRequestQueue(
   CHECK(args->GetString(0, &callback_id));
 
   if (request_coordinator_) {
-    request_coordinator_->queue()->GetRequests(base::Bind(
+    request_coordinator_->GetAllRequests(base::BindOnce(
         &OfflineInternalsUIMessageHandler::HandleRequestQueueCallback,
         weak_ptr_factory_.GetWeakPtr(), callback_id));
   } else {
@@ -295,9 +264,15 @@ void OfflineInternalsUIMessageHandler::HandleScheduleNwake(
   const base::Value* callback_id;
   CHECK(args->Get(0, &callback_id));
 
-  offline_pages::PrefetchBackgroundTask::Schedule(0);
-
-  ResolveJavascriptCallback(*callback_id, base::Value("Scheduled."));
+  if (prefetch_service_) {
+    prefetch_service_->ForceRefreshSuggestions();
+    prefetch_service_->GetPrefetchBackgroundTaskHandler()
+        ->EnsureTaskScheduled();
+    ResolveJavascriptCallback(*callback_id, base::Value("Scheduled."));
+  } else {
+    RejectJavascriptCallback(*callback_id,
+                             base::Value("No prefetch service available."));
+  }
 }
 
 void OfflineInternalsUIMessageHandler::HandleCancelNwake(
@@ -306,9 +281,25 @@ void OfflineInternalsUIMessageHandler::HandleCancelNwake(
   const base::Value* callback_id;
   CHECK(args->Get(0, &callback_id));
 
-  offline_pages::PrefetchBackgroundTask::Cancel();
+  if (prefetch_service_) {
+    prefetch_service_->GetPrefetchBackgroundTaskHandler()
+        ->CancelBackgroundTask();
+    ResolveJavascriptCallback(*callback_id, base::Value("Cancelled."));
+  } else {
+    RejectJavascriptCallback(*callback_id,
+                             base::Value("No prefetch service available."));
+  }
+}
 
-  ResolveJavascriptCallback(*callback_id, base::Value("Cancelled."));
+void OfflineInternalsUIMessageHandler::HandleShowPrefetchNotification(
+    const base::ListValue* args) {
+  AllowJavascript();
+  const base::Value* callback_id;
+  CHECK(args->Get(0, &callback_id));
+
+  offline_pages::ShowPrefetchedContentNotification(
+      GURL("https://www.example.com"));
+  ResolveJavascriptCallback(*callback_id, base::Value("Scheduled."));
 }
 
 void OfflineInternalsUIMessageHandler::HandleGeneratePageBundle(
@@ -317,18 +308,51 @@ void OfflineInternalsUIMessageHandler::HandleGeneratePageBundle(
   std::string callback_id;
   CHECK(args->GetString(0, &callback_id));
 
+  if (!prefetch_service_) {
+    RejectJavascriptCallback(base::Value(callback_id),
+                             base::Value("No prefetch service available."));
+    return;
+  }
+
   std::string data;
   CHECK(args->GetString(1, &data));
   std::vector<std::string> page_urls = base::SplitStringUsingSubstr(
       data, ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
+  std::vector<offline_pages::PrefetchURL> prefetch_urls;
+  for (auto& page_url : page_urls) {
+    // Creates a dummy prefetch URL with a bogus ID, and using the URL as the
+    // page title.
+    GURL url(page_url);
+    if (url.is_valid()) {
+      prefetch_urls.push_back(offline_pages::PrefetchURL(
+          "offline-internals", url, base::UTF8ToUTF16(page_url)));
+    }
+  }
 
-  generate_page_bundle_request_.reset(
-      new offline_pages::GeneratePageBundleRequest(
-          GetUserAgent(), "GCM ID", 1000000, page_urls, chrome::GetChannel(),
-          Profile::FromWebUI(web_ui())->GetRequestContext(),
-          base::Bind(
-              &OfflineInternalsUIMessageHandler::HandlePrefetchRequestCallback,
-              weak_ptr_factory_.GetWeakPtr(), callback_id)));
+  prefetch_service_->GetPrefetchDispatcher()->AddCandidatePrefetchURLs(
+      offline_pages::kSuggestedArticlesNamespace, prefetch_urls);
+  // Note: Static casts are needed here so that both Windows and Android can
+  // compile these printf formats.
+  std::string message =
+      base::StringPrintf("Added %zu candidate URLs.", prefetch_urls.size());
+  if (prefetch_urls.size() < page_urls.size()) {
+    size_t invalid_urls_count = page_urls.size() - prefetch_urls.size();
+    message.append(
+        base::StringPrintf(" Ignored %zu invalid URLs.", invalid_urls_count));
+  }
+  message.append("\n");
+
+  // Construct a JSON array containing all the URLs. To guard against malicious
+  // URLs that might contain special characters, we create a ListValue and then
+  // serialize it into JSON, instead of doing direct string manipulation.
+  base::ListValue urls;
+  for (const auto& prefetch_url : prefetch_urls) {
+    urls.Append(prefetch_url.url.spec());
+  }
+  std::string json;
+  base::JSONWriter::Write(urls, &json);
+  message.append(json);
+  ResolveJavascriptCallback(base::Value(callback_id), base::Value(message));
 }
 
 void OfflineInternalsUIMessageHandler::HandleGetOperation(
@@ -337,16 +361,20 @@ void OfflineInternalsUIMessageHandler::HandleGetOperation(
   std::string callback_id;
   CHECK(args->GetString(0, &callback_id));
 
+  if (!prefetch_service_) {
+    RejectJavascriptCallback(base::Value(callback_id),
+                             base::Value("No prefetch service available."));
+    return;
+  }
+
   std::string name;
   CHECK(args->GetString(1, &name));
   base::TrimWhitespaceASCII(name, base::TRIM_ALL, &name);
 
-  get_operation_request_.reset(new offline_pages::GetOperationRequest(
-      name, chrome::GetChannel(),
-      Profile::FromWebUI(web_ui())->GetRequestContext(),
-      base::Bind(
-          &OfflineInternalsUIMessageHandler::HandlePrefetchRequestCallback,
-          weak_ptr_factory_.GetWeakPtr(), callback_id)));
+  prefetch_service_->GetPrefetchDispatcher()
+      ->GCMOperationCompletedMessageReceived(name);
+  base::Value message("GetOperation will be attempted for any matching pages.");
+  ResolveJavascriptCallback(base::Value(callback_id), message);
 }
 
 void OfflineInternalsUIMessageHandler::HandleDownloadArchive(
@@ -358,21 +386,10 @@ void OfflineInternalsUIMessageHandler::HandleDownloadArchive(
 
   if (prefetch_service_) {
     prefetch_service_->GetPrefetchDownloader()->StartDownload(
-        base::GenerateGUID(), name);
+        base::GenerateGUID(), name, std::string());
   }
 }
 
-void OfflineInternalsUIMessageHandler::HandlePrefetchRequestCallback(
-    std::string callback_id,
-    offline_pages::PrefetchRequestStatus status,
-    const std::string& operation_name,
-    const std::vector<offline_pages::RenderPageInfo>& pages) {
-  ResolveJavascriptCallback(
-      base::Value(callback_id),
-      base::Value(GetStringFromPrefetchRequestStatus(status) + "\n" +
-                  operation_name + "\n" + GetStringRenderPageInfoList(pages)));
-}
-
 void OfflineInternalsUIMessageHandler::HandleSetRecordRequestQueue(
     const base::ListValue* args) {
   AllowJavascript();
@@ -391,6 +408,74 @@ void OfflineInternalsUIMessageHandler::HandleSetRecordPrefetchService(
     prefetch_service_->GetLogger()->SetIsLogging(should_record);
 }
 
+void OfflineInternalsUIMessageHandler::HandleSetLimitlessPrefetchingEnabled(
+    const base::ListValue* args) {
+  AllowJavascript();
+  PrefService* prefs = Profile::FromWebUI(web_ui())->GetPrefs();
+  DCHECK(!args->GetList().empty());
+  bool enabled = args->GetList()[0].GetBool();
+  offline_pages::prefetch_prefs::SetLimitlessPrefetchingEnabled(prefs, enabled);
+}
+
+void OfflineInternalsUIMessageHandler::HandleGetLimitlessPrefetchingEnabled(
+    const base::ListValue* args) {
+  AllowJavascript();
+  const base::Value* callback_id;
+  bool got_callback_id = args->Get(0, &callback_id);
+  DCHECK(got_callback_id);
+
+  PrefService* prefs = Profile::FromWebUI(web_ui())->GetPrefs();
+  bool enabled =
+      offline_pages::prefetch_prefs::IsLimitlessPrefetchingEnabled(prefs);
+
+  ResolveJavascriptCallback(*callback_id, base::Value(enabled));
+}
+
+void OfflineInternalsUIMessageHandler::HandleSetPrefetchTestingHeader(
+    const base::ListValue* args) {
+  AllowJavascript();
+  PrefService* prefs = Profile::FromWebUI(web_ui())->GetPrefs();
+
+  if (args->GetList().size() != 1) {
+    DLOG(ERROR) << "Expected 1 argument to setPrefetchTesting header but got "
+                << args->GetList().size();
+    return;
+  }
+  if (!args->GetList()[0].is_string()) {
+    DLOG(ERROR) << "Expected argument to be string but got "
+                << base::Value::GetTypeName(args->GetList()[0].type());
+    return;
+  }
+
+  offline_pages::prefetch_prefs::SetPrefetchTestingHeader(
+      prefs, args->GetList()[0].GetString());
+
+  if (prefetch_service_)
+    prefetch_service_->SetEnabledByServer(prefs, true);
+}
+
+void OfflineInternalsUIMessageHandler::HandleGetPrefetchTestingHeader(
+    const base::ListValue* args) {
+  AllowJavascript();
+  if (args->GetList().size() != 1) {
+    DLOG(ERROR) << "Expected 1 argument to getPrefetchTestingHeader but got "
+                << args->GetList().size();
+    return;
+  }
+  if (!args->GetList()[0].is_string()) {
+    DLOG(ERROR) << "Expected callback_id to be a string but got "
+                << base::Value::GetTypeName(args->GetList()[0].type());
+    return;
+  }
+
+  PrefService* prefs = Profile::FromWebUI(web_ui())->GetPrefs();
+  ResolveJavascriptCallback(
+      args->GetList()[0],
+      base::Value(offline_pages::prefetch_prefs::GetPrefetchTestingHeader(prefs)
+
+                      ));
+}
+
 void OfflineInternalsUIMessageHandler::HandleGetLoggingState(
     const base::ListValue* args) {
   AllowJavascript();
@@ -437,8 +522,8 @@ void OfflineInternalsUIMessageHandler::HandleGetEventLogs(
 
 void OfflineInternalsUIMessageHandler::HandleAddToRequestQueue(
     const base::ListValue* args) {
-  const base::Value* callback_id;
-  CHECK(args->Get(0, &callback_id));
+  std::string callback_id;
+  CHECK(args->GetString(0, &callback_id));
 
   if (request_coordinator_) {
     std::string url;
@@ -453,81 +538,127 @@ void OfflineInternalsUIMessageHandler::HandleAddToRequestQueue(
     params.url = GURL(url);
     params.client_id = offline_pages::ClientId(offline_pages::kAsyncNamespace,
                                                id_stream.str());
-    ResolveJavascriptCallback(
-        *callback_id,
-        base::Value(request_coordinator_->SavePageLater(params) > 0));
+    request_coordinator_->SavePageLater(
+        params,
+        base::Bind(
+            &OfflineInternalsUIMessageHandler::HandleSavePageLaterCallback,
+            weak_ptr_factory_.GetWeakPtr(), callback_id));
   } else {
-    ResolveJavascriptCallback(*callback_id, base::Value(false));
+    ResolveJavascriptCallback(base::Value(callback_id), base::Value(false));
   }
 }
 
+void OfflineInternalsUIMessageHandler::HandleSavePageLaterCallback(
+    std::string callback_id,
+    offline_pages::AddRequestResult result) {
+  ResolveJavascriptCallback(
+      base::Value(callback_id),
+      base::Value(result == offline_pages::AddRequestResult::SUCCESS));
+}
+
 void OfflineInternalsUIMessageHandler::RegisterMessages() {
   web_ui()->RegisterMessageCallback(
       "deleteSelectedPages",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleDeleteSelectedPages,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleDeleteSelectedPages,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "deleteSelectedRequests",
-      base::Bind(
+      base::BindRepeating(
           &OfflineInternalsUIMessageHandler::HandleDeleteSelectedRequests,
-          weak_ptr_factory_.GetWeakPtr()));
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "getRequestQueue",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleGetRequestQueue,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleGetRequestQueue,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "getStoredPages",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleGetStoredPages,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleGetStoredPages,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "getEventLogs",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleGetEventLogs,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(&OfflineInternalsUIMessageHandler::HandleGetEventLogs,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "setRecordRequestQueue",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleSetRecordRequestQueue,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleSetRecordRequestQueue,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "setRecordPageModel",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleSetRecordPageModel,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleSetRecordPageModel,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "setRecordPrefetchService",
-      base::Bind(
+      base::BindRepeating(
           &OfflineInternalsUIMessageHandler::HandleSetRecordPrefetchService,
-          weak_ptr_factory_.GetWeakPtr()));
+          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "setLimitlessPrefetchingEnabled",
+      base::BindRepeating(&OfflineInternalsUIMessageHandler::
+                              HandleSetLimitlessPrefetchingEnabled,
+                          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "getLimitlessPrefetchingEnabled",
+      base::BindRepeating(&OfflineInternalsUIMessageHandler::
+                              HandleGetLimitlessPrefetchingEnabled,
+                          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "setPrefetchTestingHeader",
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleSetPrefetchTestingHeader,
+          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "getPrefetchTestingHeader",
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleGetPrefetchTestingHeader,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "getLoggingState",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleGetLoggingState,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleGetLoggingState,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "addToRequestQueue",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleAddToRequestQueue,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleAddToRequestQueue,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "getNetworkStatus",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleGetNetworkStatus,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleGetNetworkStatus,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "scheduleNwake",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleScheduleNwake,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleScheduleNwake,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "cancelNwake",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleCancelNwake,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(&OfflineInternalsUIMessageHandler::HandleCancelNwake,
+                          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "showPrefetchNotification",
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleShowPrefetchNotification,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "generatePageBundle",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleGeneratePageBundle,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleGeneratePageBundle,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "getOperation",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleGetOperation,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(&OfflineInternalsUIMessageHandler::HandleGetOperation,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "downloadArchive",
-      base::Bind(&OfflineInternalsUIMessageHandler::HandleDownloadArchive,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &OfflineInternalsUIMessageHandler::HandleDownloadArchive,
+          base::Unretained(this)));
 
   // Get the offline page model associated with this web ui.
   Profile* profile = Profile::FromWebUI(web_ui());
@@ -535,8 +666,12 @@ void OfflineInternalsUIMessageHandler::RegisterMessages() {
       offline_pages::OfflinePageModelFactory::GetForBrowserContext(profile);
   request_coordinator_ =
       offline_pages::RequestCoordinatorFactory::GetForBrowserContext(profile);
-  prefetch_service_ =
-      offline_pages::PrefetchServiceFactory::GetForBrowserContext(profile);
+  prefetch_service_ = offline_pages::PrefetchServiceFactory::GetForKey(
+      profile->GetProfileKey());
+}
+
+void OfflineInternalsUIMessageHandler::OnJavascriptDisallowed() {
+  weak_ptr_factory_.InvalidateWeakPtrs();
 }
 
 }  // namespace offline_internals
