@@ -4,10 +4,12 @@
 
 #include "content/browser/accessibility/browser_accessibility_manager_android.h"
 
+#include <vector>
+
 #include "base/i18n/char_iterator.h"
 #include "content/browser/accessibility/browser_accessibility_android.h"
 #include "content/browser/accessibility/web_contents_accessibility_android.h"
-#include "content/common/accessibility_messages.h"
+#include "content/common/render_accessibility.mojom.h"
 #include "content/public/common/use_zoom_for_dsf_policy.h"
 #include "ui/accessibility/ax_role_properties.h"
 
@@ -24,14 +26,17 @@ BrowserAccessibilityManager* BrowserAccessibilityManager::Create(
 
 BrowserAccessibilityManagerAndroid::BrowserAccessibilityManagerAndroid(
     const ui::AXTreeUpdate& initial_tree,
-    WebContentsAccessibilityAndroid* web_contents_accessibility,
+    base::WeakPtr<WebContentsAccessibilityAndroid> web_contents_accessibility,
     BrowserAccessibilityDelegate* delegate,
     BrowserAccessibilityFactory* factory)
     : BrowserAccessibilityManager(delegate, factory),
-      web_contents_accessibility_(web_contents_accessibility),
+      web_contents_accessibility_(std::move(web_contents_accessibility)),
       prune_tree_for_screen_reader_(true) {
-  if (web_contents_accessibility)
-    web_contents_accessibility->set_root_manager(this);
+  // The Java layer handles the root scroll offset.
+  use_root_scroll_offsets_when_computing_bounds_ = false;
+
+  if (web_contents_accessibility_)
+    web_contents_accessibility_->set_root_manager(this);
   Initialize(initial_tree);
 }
 
@@ -103,7 +108,7 @@ void BrowserAccessibilityManagerAndroid::FireBlinkEvent(
   // Sometimes we get events on nodes in our internal accessibility tree
   // that aren't exposed on Android. Update |node| to point to the highest
   // ancestor that's a leaf node.
-  node = node->GetClosestPlatformObject();
+  node = node->PlatformGetClosestPlatformObject();
   BrowserAccessibilityAndroid* android_node =
       static_cast<BrowserAccessibilityAndroid*>(node);
 
@@ -134,7 +139,7 @@ void BrowserAccessibilityManagerAndroid::FireGeneratedEvent(
   // that aren't exposed on Android. Update |node| to point to the highest
   // ancestor that's a leaf node.
   BrowserAccessibility* original_node = node;
-  node = node->GetClosestPlatformObject();
+  node = node->PlatformGetClosestPlatformObject();
   BrowserAccessibilityAndroid* android_node =
       static_cast<BrowserAccessibilityAndroid*>(node);
 
@@ -143,7 +148,7 @@ void BrowserAccessibilityManagerAndroid::FireGeneratedEvent(
   // character with a dot after a short pause. On Android we don't want to
   // fire an event for those changes, but we do want to make sure our internal
   // state is correct, so we call OnDataChanged() and then return.
-  if (android_node->IsPassword() && original_node != node) {
+  if (android_node->IsPasswordField() && original_node != node) {
     android_node->OnDataChanged();
     return;
   }
@@ -191,7 +196,7 @@ void BrowserAccessibilityManagerAndroid::FireGeneratedEvent(
       break;
     }
     case ui::AXEventGenerator::Event::VALUE_CHANGED:
-      if (android_node->IsEditableText() && GetFocus() == node) {
+      if (android_node->IsTextField() && GetFocus() == node) {
         wcax->HandleEditableTextChanged(android_node->unique_id());
       } else if (android_node->IsSlider()) {
         wcax->HandleSliderChanged(android_node->unique_id());
@@ -236,6 +241,7 @@ void BrowserAccessibilityManagerAndroid::FireGeneratedEvent(
     case ui::AXEventGenerator::Event::NAME_CHANGED:
     case ui::AXEventGenerator::Event::OTHER_ATTRIBUTE_CHANGED:
     case ui::AXEventGenerator::Event::PLACEHOLDER_CHANGED:
+    case ui::AXEventGenerator::Event::PORTAL_ACTIVATED:
     case ui::AXEventGenerator::Event::POSITION_IN_SET_CHANGED:
     case ui::AXEventGenerator::Event::READONLY_CHANGED:
     case ui::AXEventGenerator::Event::RELATED_NODE_CHANGED:
@@ -258,19 +264,19 @@ void BrowserAccessibilityManagerAndroid::FireGeneratedEvent(
 }
 
 void BrowserAccessibilityManagerAndroid::SendLocationChangeEvents(
-    const std::vector<AccessibilityHostMsg_LocationChangeParams>& params) {
+    const std::vector<mojom::LocationChangesPtr>& changes) {
   // Android is not very efficient at handling notifications, and location
   // changes in particular are frequent and not time-critical. If a lot of
   // nodes changed location, just send a single notification after a short
   // delay (to batch them), rather than lots of individual notifications.
-  if (params.size() > 3) {
+  if (changes.size() > 3) {
     auto* wcax = GetWebContentsAXFromRootManager();
     if (!wcax)
       return;
     wcax->SendDelayedWindowContentChangedEvent();
     return;
   }
-  BrowserAccessibilityManager::SendLocationChangeEvents(params);
+  BrowserAccessibilityManager::SendLocationChangeEvents(changes);
 }
 
 bool BrowserAccessibilityManagerAndroid::NextAtGranularity(
@@ -287,8 +293,9 @@ bool BrowserAccessibilityManagerAndroid::NextAtGranularity(
       base::i18n::UTF16CharIterator iter(text.data(), text.size());
       while (!iter.end() && iter.array_pos() <= cursor_index)
         iter.Advance();
-      *start_index = iter.array_pos();
       *end_index = iter.array_pos();
+      iter.Rewind();
+      *start_index = iter.array_pos();
       break;
     }
     case ANDROID_ACCESSIBILITY_NODE_INFO_MOVEMENT_GRANULARITY_WORD:
@@ -335,7 +342,7 @@ bool BrowserAccessibilityManagerAndroid::PreviousAtGranularity(
         iter.Advance();
       }
       *start_index = previous_index;
-      *end_index = previous_index;
+      *end_index = iter.array_pos();
       break;
     }
     case ANDROID_ACCESSIBILITY_NODE_INFO_MOVEMENT_GRANULARITY_WORD:
@@ -364,6 +371,15 @@ bool BrowserAccessibilityManagerAndroid::PreviousAtGranularity(
   return true;
 }
 
+void BrowserAccessibilityManagerAndroid::ClearNodeInfoCacheForGivenId(
+    int32_t unique_id) {
+  WebContentsAccessibilityAndroid* wcax = GetWebContentsAXFromRootManager();
+  if (!wcax)
+    return;
+
+  wcax->ClearNodeInfoCacheForGivenId(unique_id);
+}
+
 bool BrowserAccessibilityManagerAndroid::OnHoverEvent(
     const ui::MotionEventAndroid& event) {
   WebContentsAccessibilityAndroid* wcax = GetWebContentsAXFromRootManager();
@@ -378,7 +394,7 @@ void BrowserAccessibilityManagerAndroid::HandleHoverEvent(
 
   // First walk up to the nearest platform node, in case this node isn't
   // even exposed on the platform.
-  node = node->GetClosestPlatformObject();
+  node = node->PlatformGetClosestPlatformObject();
 
   // If this node is uninteresting and just a wrapper around a sole
   // interesting descendant, prefer that descendant instead.
@@ -400,11 +416,19 @@ void BrowserAccessibilityManagerAndroid::HandleHoverEvent(
     wcax->HandleHover(android_node->unique_id());
 }
 
-gfx::Rect BrowserAccessibilityManagerAndroid::GetViewBounds() {
+gfx::Rect BrowserAccessibilityManagerAndroid::GetViewBoundsInScreenCoordinates()
+    const {
   // We have to take the device scale factor into account on Android.
   BrowserAccessibilityDelegate* delegate = GetDelegateFromRootManager();
   if (delegate) {
     gfx::Rect bounds = delegate->AccessibilityGetViewBounds();
+
+    // http://www.chromium.org/developers/design-documents/blink-coordinate-spaces
+    // The bounds returned by the delegate are always in device-independent
+    // pixels (DIPs), meaning physical pixels divided by device scale factor
+    // (DSF). However, if UseZoomForDSF is enabled, then Blink does not apply
+    // DSF when going from physical to screen pixels. In that case, we need to
+    // multiply DSF back in to get to Blink's notion of "screen pixels."
     if (IsUseZoomForDSFEnabled() && device_scale_factor() > 0.0 &&
         device_scale_factor() != 1.0)
       bounds = ScaleToEnclosingRect(bounds, device_scale_factor());
@@ -413,6 +437,17 @@ gfx::Rect BrowserAccessibilityManagerAndroid::GetViewBounds() {
   return gfx::Rect();
 }
 
+void BrowserAccessibilityManagerAndroid::OnNodeWillBeDeleted(ui::AXTree* tree,
+                                                             ui::AXNode* node) {
+  BrowserAccessibility* wrapper = GetFromAXNode(node);
+  BrowserAccessibilityAndroid* android_node =
+      static_cast<BrowserAccessibilityAndroid*>(wrapper);
+
+  ClearNodeInfoCacheForGivenId(android_node->unique_id());
+
+  BrowserAccessibilityManager::OnNodeWillBeDeleted(tree, node);
+}
+
 void BrowserAccessibilityManagerAndroid::OnAtomicUpdateFinished(
     ui::AXTree* tree,
     bool root_changed,
@@ -428,17 +463,11 @@ void BrowserAccessibilityManagerAndroid::OnAtomicUpdateFinished(
   }
 }
 
-bool BrowserAccessibilityManagerAndroid::
-    UseRootScrollOffsetsWhenComputingBounds() {
-  // The Java layer handles the root scroll offset.
-  return false;
-}
-
 WebContentsAccessibilityAndroid*
 BrowserAccessibilityManagerAndroid::GetWebContentsAXFromRootManager() {
   BrowserAccessibility* parent_node = GetParentNodeFromParentTree();
   if (!parent_node)
-    return web_contents_accessibility_;
+    return web_contents_accessibility_.get();
 
   auto* parent_manager =
       static_cast<BrowserAccessibilityManagerAndroid*>(parent_node->manager());
