@@ -5,16 +5,18 @@
 #include "content/browser/devtools/protocol/service_worker_handler.h"
 
 #include "base/bind.h"
+#include "base/bind_helpers.h"
+#include "base/containers/flat_set.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/containers/flat_set.h"
-#include "content/browser/background_sync/background_sync_context.h"
+#include "base/task/post_task.h"
+#include "content/browser/background_sync/background_sync_context_impl.h"
 #include "content/browser/background_sync/background_sync_manager.h"
 #include "content/browser/devtools/service_worker_devtools_agent_host.h"
 #include "content/browser/devtools/service_worker_devtools_manager.h"
+#include "content/browser/devtools/shared_worker_devtools_manager.h"
 #include "content/browser/frame_host/frame_tree.h"
 #include "content/browser/frame_host/frame_tree_node.h"
-#include "content/browser/frame_host/render_frame_host_impl.h"
 #include "content/browser/service_worker/embedded_worker_status.h"
 #include "content/browser/service_worker/service_worker_context_watcher.h"
 #include "content/browser/service_worker/service_worker_context_wrapper.h"
@@ -22,14 +24,16 @@
 #include "content/browser/storage_partition_impl_map.h"
 #include "content/common/service_worker/service_worker_utils.h"
 #include "content/public/browser/browser_context.h"
+#include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/devtools_agent_host.h"
-#include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
+#include "content/public/browser/service_worker_context.h"
 #include "content/public/browser/storage_partition.h"
 #include "content/public/browser/web_contents.h"
-#include "content/public/common/push_event_payload.h"
-#include "content/public/common/push_messaging_status.mojom.h"
+#include "third_party/blink/public/mojom/push_messaging/push_messaging_status.mojom.h"
+#include "third_party/blink/public/mojom/service_worker/service_worker_container_type.mojom.h"
+#include "third_party/blink/public/mojom/service_worker/service_worker_object.mojom.h"
 #include "url/gurl.h"
 
 namespace content {
@@ -37,19 +41,6 @@ namespace protocol {
 
 namespace {
 
-void ResultNoOp(bool success) {
-}
-
-void StatusNoOp(ServiceWorkerStatusCode status) {
-}
-
-void StatusNoOpKeepingRegistration(
-    scoped_refptr<content::ServiceWorkerRegistration> protect,
-    ServiceWorkerStatusCode status) {
-}
-
-void PushDeliveryNoOp(mojom::PushDeliveryStatus status) {}
-
 const std::string GetVersionRunningStatusString(
     EmbeddedWorkerStatus running_status) {
   switch (running_status) {
@@ -88,43 +79,49 @@ const std::string GetVersionStatusString(
   return std::string();
 }
 
-void StopServiceWorkerOnIO(scoped_refptr<ServiceWorkerContextWrapper> context,
-                           int64_t version_id) {
+void StopServiceWorkerOnCoreThread(
+    scoped_refptr<ServiceWorkerContextWrapper> context,
+    int64_t version_id) {
   if (content::ServiceWorkerVersion* version =
           context->GetLiveVersion(version_id)) {
-    version->StopWorker(base::Bind(&StatusNoOp));
+    version->StopWorker(base::DoNothing());
   }
 }
 
-void GetDevToolsRouteInfoOnIO(
+void GetDevToolsRouteInfoOnCoreThread(
     scoped_refptr<ServiceWorkerContextWrapper> context,
     int64_t version_id,
-    const base::Callback<void(int, int)>& callback) {
+    base::OnceCallback<void(int, int)> callback) {
   if (content::ServiceWorkerVersion* version =
           context->GetLiveVersion(version_id)) {
-    BrowserThread::PostTask(
-        BrowserThread::UI, FROM_HERE,
-        base::Bind(
-            callback, version->embedded_worker()->process_id(),
+    RunOrPostTaskOnThread(
+        FROM_HERE, BrowserThread::UI,
+        base::BindOnce(
+            std::move(callback), version->embedded_worker()->process_id(),
             version->embedded_worker()->worker_devtools_agent_route_id()));
   }
 }
 
+Response CreateDomainNotEnabledErrorResponse() {
+  return Response::ServerError("ServiceWorker domain not enabled");
+}
+
 Response CreateContextErrorResponse() {
-  return Response::Error("Could not connect to the context");
+  return Response::ServerError("Could not connect to the context");
 }
 
 Response CreateInvalidVersionIdErrorResponse() {
   return Response::InvalidParams("Invalid version ID");
 }
 
-void DidFindRegistrationForDispatchSyncEventOnIO(
-    scoped_refptr<BackgroundSyncContext> sync_context,
+void DidFindRegistrationForDispatchSyncEventOnCoreThread(
+    scoped_refptr<BackgroundSyncContextImpl> sync_context,
     const std::string& tag,
     bool last_chance,
-    ServiceWorkerStatusCode status,
+    blink::ServiceWorkerStatusCode status,
     scoped_refptr<content::ServiceWorkerRegistration> registration) {
-  if (status != SERVICE_WORKER_OK || !registration->active_version())
+  if (status != blink::ServiceWorkerStatusCode::kOk ||
+      !registration->active_version())
     return;
   BackgroundSyncManager* background_sync_manager =
       sync_context->background_sync_manager();
@@ -132,30 +129,62 @@ void DidFindRegistrationForDispatchSyncEventOnIO(
       registration->active_version());
   // Keep the registration while dispatching the sync event.
   background_sync_manager->EmulateDispatchSyncEvent(
-      tag, std::move(version), last_chance,
-      base::Bind(&StatusNoOpKeepingRegistration, std::move(registration)));
+      tag, std::move(version), last_chance, base::DoNothing());
 }
 
-void DispatchSyncEventOnIO(scoped_refptr<ServiceWorkerContextWrapper> context,
-                           scoped_refptr<BackgroundSyncContext> sync_context,
-                           const GURL& origin,
-                           int64_t registration_id,
-                           const std::string& tag,
-                           bool last_chance) {
+void DidFindRegistrationForDispatchPeriodicSyncEventOnCoreThread(
+    scoped_refptr<BackgroundSyncContextImpl> sync_context,
+    const std::string& tag,
+    blink::ServiceWorkerStatusCode status,
+    scoped_refptr<content::ServiceWorkerRegistration> registration) {
+  if (status != blink::ServiceWorkerStatusCode::kOk ||
+      !registration->active_version()) {
+    return;
+  }
+
+  BackgroundSyncManager* background_sync_manager =
+      sync_context->background_sync_manager();
+  scoped_refptr<content::ServiceWorkerVersion> version(
+      registration->active_version());
+  // Keep the registration while dispatching the sync event.
+  background_sync_manager->EmulateDispatchPeriodicSyncEvent(
+      tag, std::move(version), base::DoNothing());
+}
+
+void DispatchSyncEventOnCoreThread(
+    scoped_refptr<ServiceWorkerContextWrapper> context,
+    scoped_refptr<BackgroundSyncContextImpl> sync_context,
+    const GURL& origin,
+    int64_t registration_id,
+    const std::string& tag,
+    bool last_chance) {
   context->FindReadyRegistrationForId(
       registration_id, origin,
-      base::Bind(&DidFindRegistrationForDispatchSyncEventOnIO, sync_context,
-                 tag, last_chance));
+      base::BindOnce(&DidFindRegistrationForDispatchSyncEventOnCoreThread,
+                     sync_context, tag, last_chance));
+}
+
+void DispatchPeriodicSyncEventOnCoreThread(
+    scoped_refptr<ServiceWorkerContextWrapper> context,
+    scoped_refptr<BackgroundSyncContextImpl> sync_context,
+    const GURL& origin,
+    int64_t registration_id,
+    const std::string& tag) {
+  context->FindReadyRegistrationForId(
+      registration_id, origin,
+      base::BindOnce(
+          &DidFindRegistrationForDispatchPeriodicSyncEventOnCoreThread,
+          sync_context, tag));
 }
 
 }  // namespace
 
-ServiceWorkerHandler::ServiceWorkerHandler()
+ServiceWorkerHandler::ServiceWorkerHandler(bool allow_inspect_worker)
     : DevToolsDomainHandler(ServiceWorker::Metainfo::domainName),
+      allow_inspect_worker_(allow_inspect_worker),
       enabled_(false),
-      render_frame_host_(nullptr),
-      weak_factory_(this) {
-}
+      browser_context_(nullptr),
+      storage_partition_(nullptr) {}
 
 ServiceWorkerHandler::~ServiceWorkerHandler() {
 }
@@ -165,119 +194,138 @@ void ServiceWorkerHandler::Wire(UberDispatcher* dispatcher) {
   ServiceWorker::Dispatcher::wire(dispatcher, this);
 }
 
-void ServiceWorkerHandler::SetRenderFrameHost(
-    RenderFrameHostImpl* render_frame_host) {
-  render_frame_host_ = render_frame_host;
+void ServiceWorkerHandler::SetRenderer(int process_host_id,
+                                       RenderFrameHostImpl* frame_host) {
+  RenderProcessHost* process_host = RenderProcessHost::FromID(process_host_id);
   // Do not call UpdateHosts yet, wait for load to commit.
-  if (!render_frame_host) {
+  if (!process_host) {
     ClearForceUpdate();
     context_ = nullptr;
     return;
   }
-  StoragePartition* partition = BrowserContext::GetStoragePartition(
-      render_frame_host->GetProcess()->GetBrowserContext(),
-      render_frame_host->GetSiteInstance());
-  DCHECK(partition);
+
+  storage_partition_ =
+      static_cast<StoragePartitionImpl*>(process_host->GetStoragePartition());
+  DCHECK(storage_partition_);
+  browser_context_ = process_host->GetBrowserContext();
   context_ = static_cast<ServiceWorkerContextWrapper*>(
-      partition->GetServiceWorkerContext());
+      storage_partition_->GetServiceWorkerContext());
 }
 
 Response ServiceWorkerHandler::Enable() {
   if (enabled_)
-    return Response::OK();
+    return Response::Success();
   if (!context_)
     return CreateContextErrorResponse();
   enabled_ = true;
 
-  context_watcher_ = new ServiceWorkerContextWatcher(
-      context_, base::Bind(&ServiceWorkerHandler::OnWorkerRegistrationUpdated,
-                           weak_factory_.GetWeakPtr()),
-      base::Bind(&ServiceWorkerHandler::OnWorkerVersionUpdated,
-                 weak_factory_.GetWeakPtr()),
-      base::Bind(&ServiceWorkerHandler::OnErrorReported,
-                 weak_factory_.GetWeakPtr()));
+  context_watcher_ = base::MakeRefCounted<ServiceWorkerContextWatcher>(
+      context_,
+      base::BindRepeating(&ServiceWorkerHandler::OnWorkerRegistrationUpdated,
+                          weak_factory_.GetWeakPtr()),
+      base::BindRepeating(&ServiceWorkerHandler::OnWorkerVersionUpdated,
+                          weak_factory_.GetWeakPtr()),
+      base::BindRepeating(&ServiceWorkerHandler::OnErrorReported,
+                          weak_factory_.GetWeakPtr()));
   context_watcher_->Start();
 
-  return Response::OK();
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::Disable() {
   if (!enabled_)
-    return Response::OK();
+    return Response::Success();
   enabled_ = false;
 
   ClearForceUpdate();
   DCHECK(context_watcher_);
   context_watcher_->Stop();
   context_watcher_ = nullptr;
-  return Response::OK();
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::Unregister(const std::string& scope_url) {
   if (!enabled_)
-    return Response::OK();
+    return CreateDomainNotEnabledErrorResponse();
   if (!context_)
     return CreateContextErrorResponse();
-  context_->UnregisterServiceWorker(GURL(scope_url), base::Bind(&ResultNoOp));
-  return Response::OK();
+  context_->UnregisterServiceWorker(GURL(scope_url), base::DoNothing());
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::StartWorker(const std::string& scope_url) {
   if (!enabled_)
-    return Response::OK();
+    return CreateDomainNotEnabledErrorResponse();
   if (!context_)
     return CreateContextErrorResponse();
-  context_->StartServiceWorker(GURL(scope_url), base::Bind(&StatusNoOp));
-  return Response::OK();
+  context_->StartServiceWorker(GURL(scope_url), base::DoNothing());
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::SkipWaiting(const std::string& scope_url) {
   if (!enabled_)
-    return Response::OK();
+    return CreateDomainNotEnabledErrorResponse();
   if (!context_)
     return CreateContextErrorResponse();
   context_->SkipWaitingWorker(GURL(scope_url));
-  return Response::OK();
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::StopWorker(const std::string& version_id) {
   if (!enabled_)
-    return Response::OK();
+    return CreateDomainNotEnabledErrorResponse();
   if (!context_)
     return CreateContextErrorResponse();
   int64_t id = 0;
   if (!base::StringToInt64(version_id, &id))
     return CreateInvalidVersionIdErrorResponse();
-  BrowserThread::PostTask(BrowserThread::IO, FROM_HERE,
-                          base::Bind(&StopServiceWorkerOnIO, context_, id));
-  return Response::OK();
+  RunOrPostTaskOnThread(
+      FROM_HERE, ServiceWorkerContext::GetCoreThreadId(),
+      base::BindOnce(&StopServiceWorkerOnCoreThread, context_, id));
+  return Response::Success();
+}
+
+void ServiceWorkerHandler::StopAllWorkers(
+    std::unique_ptr<StopAllWorkersCallback> callback) {
+  if (!enabled_) {
+    callback->sendFailure(CreateDomainNotEnabledErrorResponse());
+    return;
+  }
+  if (!context_) {
+    callback->sendFailure(CreateContextErrorResponse());
+    return;
+  }
+  context_->StopAllServiceWorkers(base::BindOnce(
+      &StopAllWorkersCallback::sendSuccess, std::move(callback)));
 }
 
 Response ServiceWorkerHandler::UpdateRegistration(
     const std::string& scope_url) {
   if (!enabled_)
-    return Response::OK();
+    return CreateDomainNotEnabledErrorResponse();
   if (!context_)
     return CreateContextErrorResponse();
   context_->UpdateRegistration(GURL(scope_url));
-  return Response::OK();
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::InspectWorker(const std::string& version_id) {
   if (!enabled_)
-    return Response::OK();
+    return CreateDomainNotEnabledErrorResponse();
   if (!context_)
     return CreateContextErrorResponse();
-
-  int64_t id = kInvalidServiceWorkerVersionId;
+  if (!allow_inspect_worker_)
+    return Response::ServerError("Permission denied");
+  int64_t id = blink::mojom::kInvalidServiceWorkerVersionId;
   if (!base::StringToInt64(version_id, &id))
     return CreateInvalidVersionIdErrorResponse();
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      base::Bind(&GetDevToolsRouteInfoOnIO, context_, id,
-                 base::Bind(&ServiceWorkerHandler::OpenNewDevToolsWindow,
-                            weak_factory_.GetWeakPtr())));
-  return Response::OK();
+  RunOrPostTaskOnThread(
+      FROM_HERE, ServiceWorkerContext::GetCoreThreadId(),
+      base::BindOnce(
+          &GetDevToolsRouteInfoOnCoreThread, context_, id,
+          base::BindOnce(&ServiceWorkerHandler::OpenNewDevToolsWindow,
+                         weak_factory_.GetWeakPtr())));
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::SetForceUpdateOnPageLoad(
@@ -285,7 +333,7 @@ Response ServiceWorkerHandler::SetForceUpdateOnPageLoad(
   if (!context_)
     return CreateContextErrorResponse();
   context_->SetForceUpdateOnPageLoad(force_update_on_page_load);
-  return Response::OK();
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::DeliverPushMessage(
@@ -293,19 +341,21 @@ Response ServiceWorkerHandler::DeliverPushMessage(
     const std::string& registration_id,
     const std::string& data) {
   if (!enabled_)
-    return Response::OK();
-  if (!render_frame_host_)
+    return CreateDomainNotEnabledErrorResponse();
+  if (!browser_context_)
     return CreateContextErrorResponse();
   int64_t id = 0;
   if (!base::StringToInt64(registration_id, &id))
     return CreateInvalidVersionIdErrorResponse();
-  PushEventPayload payload;
+  base::Optional<std::string> payload;
   if (data.size() > 0)
-    payload.setData(data);
+    payload = data;
   BrowserContext::DeliverPushMessage(
-      render_frame_host_->GetProcess()->GetBrowserContext(), GURL(origin), id,
-      payload, base::Bind(&PushDeliveryNoOp));
-  return Response::OK();
+      browser_context_, GURL(origin), id, /* push_message_id= */ std::string(),
+      std::move(payload),
+      base::BindOnce([](blink::mojom::PushDeliveryStatus status) {}));
+
+  return Response::Success();
 }
 
 Response ServiceWorkerHandler::DispatchSyncEvent(
@@ -314,24 +364,44 @@ Response ServiceWorkerHandler::DispatchSyncEvent(
     const std::string& tag,
     bool last_chance) {
   if (!enabled_)
-    return Response::OK();
-  if (!render_frame_host_)
+    return CreateDomainNotEnabledErrorResponse();
+  if (!storage_partition_)
     return CreateContextErrorResponse();
   int64_t id = 0;
   if (!base::StringToInt64(registration_id, &id))
     return CreateInvalidVersionIdErrorResponse();
 
-  StoragePartitionImpl* partition =
-      static_cast<StoragePartitionImpl*>(BrowserContext::GetStoragePartition(
-          render_frame_host_->GetProcess()->GetBrowserContext(),
-          render_frame_host_->GetSiteInstance()));
-  BackgroundSyncContext* sync_context = partition->GetBackgroundSyncContext();
+  BackgroundSyncContextImpl* sync_context =
+      storage_partition_->GetBackgroundSyncContext();
 
-  BrowserThread::PostTask(BrowserThread::IO, FROM_HERE,
-                          base::Bind(&DispatchSyncEventOnIO, context_,
-                                     make_scoped_refptr(sync_context),
-                                     GURL(origin), id, tag, last_chance));
-  return Response::OK();
+  RunOrPostTaskOnThread(FROM_HERE, ServiceWorkerContext::GetCoreThreadId(),
+                        base::BindOnce(&DispatchSyncEventOnCoreThread, context_,
+                                       base::WrapRefCounted(sync_context),
+                                       GURL(origin), id, tag, last_chance));
+  return Response::Success();
+}
+
+Response ServiceWorkerHandler::DispatchPeriodicSyncEvent(
+    const std::string& origin,
+    const std::string& registration_id,
+    const std::string& tag) {
+  if (!enabled_)
+    return CreateDomainNotEnabledErrorResponse();
+  if (!storage_partition_)
+    return CreateContextErrorResponse();
+  int64_t id = 0;
+  if (!base::StringToInt64(registration_id, &id))
+    return CreateInvalidVersionIdErrorResponse();
+
+  BackgroundSyncContextImpl* sync_context =
+      storage_partition_->GetBackgroundSyncContext();
+
+  RunOrPostTaskOnThread(
+      FROM_HERE, ServiceWorkerContext::GetCoreThreadId(),
+      base::BindOnce(&DispatchPeriodicSyncEventOnCoreThread, context_,
+                     base::WrapRefCounted(sync_context), GURL(origin), id,
+                     tag));
+  return Response::Success();
 }
 
 void ServiceWorkerHandler::OpenNewDevToolsWindow(int process_id,
@@ -347,16 +417,16 @@ void ServiceWorkerHandler::OpenNewDevToolsWindow(int process_id,
 void ServiceWorkerHandler::OnWorkerRegistrationUpdated(
     const std::vector<ServiceWorkerRegistrationInfo>& registrations) {
   using Registration = ServiceWorker::ServiceWorkerRegistration;
-  std::unique_ptr<protocol::Array<Registration>> result =
-      protocol::Array<Registration>::create();
+  auto result = std::make_unique<protocol::Array<Registration>>();
   for (const auto& registration : registrations) {
-    result->addItem(Registration::Create()
-        .SetRegistrationId(
-            base::Int64ToString(registration.registration_id))
-        .SetScopeURL(registration.pattern.spec())
-        .SetIsDeleted(registration.delete_flag ==
-                      ServiceWorkerRegistrationInfo::IS_DELETED)
-        .Build());
+    result->emplace_back(
+        Registration::Create()
+            .SetRegistrationId(
+                base::NumberToString(registration.registration_id))
+            .SetScopeURL(registration.scope.spec())
+            .SetIsDeleted(registration.delete_flag ==
+                          ServiceWorkerRegistrationInfo::IS_DELETED)
+            .Build());
   }
   frontend_->WorkerRegistrationUpdated(std::move(result));
 }
@@ -364,15 +434,15 @@ void ServiceWorkerHandler::OnWorkerRegistrationUpdated(
 void ServiceWorkerHandler::OnWorkerVersionUpdated(
     const std::vector<ServiceWorkerVersionInfo>& versions) {
   using Version = ServiceWorker::ServiceWorkerVersion;
-  std::unique_ptr<protocol::Array<Version>> result =
-      protocol::Array<Version>::create();
+  auto result = std::make_unique<protocol::Array<Version>>();
   for (const auto& version : versions) {
     base::flat_set<std::string> client_set;
 
     for (const auto& client : version.clients) {
-      if (client.second.type == SERVICE_WORKER_PROVIDER_FOR_WINDOW) {
-        // PlzNavigate: a navigation may not yet be associated with a
-        // RenderFrameHost. Use the |web_contents_getter| instead.
+      if (client.second.type ==
+          blink::mojom::ServiceWorkerContainerType::kForWindow) {
+        // A navigation may not yet be associated with a RenderFrameHost. Use
+        // the |web_contents_getter| instead.
         WebContents* web_contents =
             client.second.web_contents_getter
                 ? client.second.web_contents_getter.Run()
@@ -384,34 +454,24 @@ void ServiceWorkerHandler::OnWorkerVersionUpdated(
           continue;
         client_set.insert(
             DevToolsAgentHost::GetOrCreateFor(web_contents)->GetId());
-      } else if (client.second.type ==
-                 SERVICE_WORKER_PROVIDER_FOR_SHARED_WORKER) {
-        scoped_refptr<DevToolsAgentHost> agent_host(
-            DevToolsAgentHost::GetForWorker(client.second.process_id,
-                                            client.second.route_id));
-        if (agent_host)
-          client_set.insert(agent_host->GetId());
       }
     }
-    std::unique_ptr<protocol::Array<std::string>> clients =
-        protocol::Array<std::string>::create();
-    for (auto& c : client_set)
-      clients->addItem(c);
+    auto clients = std::make_unique<protocol::Array<std::string>>();
+    for (std::string& client : client_set)
+      clients->emplace_back(std::move(client));
 
-    std::unique_ptr<Version> version_value = Version::Create()
-        .SetVersionId(base::Int64ToString(version.version_id))
-        .SetRegistrationId(
-            base::Int64ToString(version.registration_id))
-        .SetScriptURL(version.script_url.spec())
-        .SetRunningStatus(
-            GetVersionRunningStatusString(version.running_status))
-        .SetStatus(GetVersionStatusString(version.status))
-        .SetScriptLastModified(
-            version.script_last_modified.ToDoubleT())
-        .SetScriptResponseTime(
-            version.script_response_time.ToDoubleT())
-        .SetControlledClients(std::move(clients))
-        .Build();
+    std::unique_ptr<Version> version_value =
+        Version::Create()
+            .SetVersionId(base::NumberToString(version.version_id))
+            .SetRegistrationId(base::NumberToString(version.registration_id))
+            .SetScriptURL(version.script_url.spec())
+            .SetRunningStatus(
+                GetVersionRunningStatusString(version.running_status))
+            .SetStatus(GetVersionStatusString(version.status))
+            .SetScriptLastModified(version.script_last_modified.ToDoubleT())
+            .SetScriptResponseTime(version.script_response_time.ToDoubleT())
+            .SetControlledClients(std::move(clients))
+            .Build();
     scoped_refptr<DevToolsAgentHostImpl> host(
         ServiceWorkerDevToolsManager::GetInstance()
             ->GetDevToolsAgentHostForWorker(
@@ -419,7 +479,7 @@ void ServiceWorkerHandler::OnWorkerVersionUpdated(
                 version.devtools_agent_route_id));
     if (host)
       version_value->SetTargetId(host->GetId());
-    result->addItem(std::move(version_value));
+    result->emplace_back(std::move(version_value));
   }
   frontend_->WorkerVersionUpdated(std::move(result));
 }
@@ -431,8 +491,8 @@ void ServiceWorkerHandler::OnErrorReported(
   frontend_->WorkerErrorReported(
       ServiceWorker::ServiceWorkerErrorMessage::Create()
           .SetErrorMessage(base::UTF16ToUTF8(info.error_message))
-          .SetRegistrationId(base::Int64ToString(registration_id))
-          .SetVersionId(base::Int64ToString(version_id))
+          .SetRegistrationId(base::NumberToString(registration_id))
+          .SetVersionId(base::NumberToString(version_id))
           .SetSourceURL(info.source_url.spec())
           .SetLineNumber(info.line_number)
           .SetColumnNumber(info.column_number)
