@@ -9,11 +9,15 @@
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
 #include "content/public/browser/web_contents.h"
+#include "content/public/common/content_features.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/content_browser_test_utils.h"
 #include "content/shell/browser/shell.h"
+#include "content/shell/common/shell_switches.h"
+#include "media/audio/audio_features.h"
 #include "media/base/media_switches.h"
+#include "media/base/supported_types.h"
 #include "media/base/test_data_util.h"
 #include "media/media_buildflags.h"
 #include "net/test/embedded_test_server/embedded_test_server.h"
@@ -30,9 +34,26 @@ void MediaBrowserTest::SetUpCommandLine(base::CommandLine* command_line) {
   command_line->AppendSwitchASCII(
       switches::kAutoplayPolicy,
       switches::autoplay::kNoUserGestureRequiredPolicy);
-  // Disable fallback after decode error to avoid unexpected test pass on the
-  // fallback path.
-  scoped_feature_list_.InitAndDisableFeature(media::kFallbackAfterDecodeError);
+  command_line->AppendSwitch(switches::kExposeInternalsForTesting);
+#if defined(OS_LINUX)
+  // Due to problems with PulseAudio failing to start, use a fake audio
+  // stream. crbug.com/1047655#c70
+  command_line->AppendSwitch(switches::kDisableAudioOutput);
+#endif
+
+  std::vector<base::Feature> enabled_features = {
+#if defined(OS_ANDROID)
+    features::kLogJsConsoleMessages,
+#endif
+  };
+
+  std::vector<base::Feature> disabled_features = {
+    // Disable fallback after decode error to avoid unexpected test pass on
+    // the fallback path.
+    media::kFallbackAfterDecodeError,
+  };
+
+  scoped_feature_list_.InitWithFeatures(enabled_features, disabled_features);
 }
 
 void MediaBrowserTest::RunMediaTestPage(const std::string& html_page,
@@ -61,7 +82,7 @@ std::string MediaBrowserTest::RunTest(const GURL& gurl,
   TitleWatcher title_watcher(shell()->web_contents(),
                              base::ASCIIToUTF16(expected_title));
   AddTitlesToAwait(&title_watcher);
-  NavigateToURL(shell(), gurl);
+  EXPECT_TRUE(NavigateToURL(shell(), gurl));
   base::string16 result = title_watcher.WaitAndGetTitle();
 
   CleanupTest();
@@ -76,7 +97,7 @@ void MediaBrowserTest::CleanupTest() {
   TitleWatcher clean_title_watcher(shell()->web_contents(), cleaner_title);
   GURL cleaner_url = content::GetFileUrlWithQuery(
       media::GetTestDataFilePath("cleaner.html"), "");
-  NavigateToURL(shell(), cleaner_url);
+  EXPECT_TRUE(NavigateToURL(shell(), cleaner_url));
   base::string16 cleaner_result = clean_title_watcher.WaitAndGetTitle();
   EXPECT_EQ(cleaner_result, cleaner_title);
 #endif
@@ -134,13 +155,13 @@ class MediaTest : public testing::WithParamInterface<bool>,
   }
 
   void RunVideoSizeTest(const char* media_file, int width, int height) {
-    std::string expected;
-    expected += base::NumberToString(width);
-    expected += " ";
-    expected += base::NumberToString(height);
+    std::string expected_title = std::string(media::kEnded) + " " +
+                                 base::NumberToString(width) + " " +
+                                 base::NumberToString(height);
     base::StringPairs query_params;
     query_params.emplace_back("video", media_file);
-    RunMediaTestPage("player.html", query_params, expected, false);
+    query_params.emplace_back("sizetest", "true");
+    RunMediaTestPage("player.html", query_params, expected_title, false);
   }
 };
 
@@ -160,15 +181,15 @@ IN_PROC_BROWSER_TEST_P(MediaTest, VideoBearWebm) {
 }
 
 IN_PROC_BROWSER_TEST_P(MediaTest, AudioBearOpusWebm) {
-  PlayVideo("bear-opus.webm", GetParam());
+  PlayAudio("bear-opus.webm", GetParam());
 }
 
 IN_PROC_BROWSER_TEST_P(MediaTest, AudioBearOpusMp4) {
-  PlayVideo("bear-opus.mp4", GetParam());
+  PlayAudio("bear-opus.mp4", GetParam());
 }
 
 IN_PROC_BROWSER_TEST_P(MediaTest, AudioBearOpusOgg) {
-  PlayVideo("bear-opus.ogg", GetParam());
+  PlayAudio("bear-opus.ogg", GetParam());
 }
 
 IN_PROC_BROWSER_TEST_P(MediaTest, VideoBearSilentWebm) {
@@ -199,11 +220,11 @@ IN_PROC_BROWSER_TEST_P(MediaTest, AudioBearFlac192kHzMp4) {
 }
 
 IN_PROC_BROWSER_TEST_P(MediaTest, VideoBearMovPcmS16be) {
-  PlayVideo("bear_pcm_s16be.mov", GetParam());
+  PlayAudio("bear_pcm_s16be.mov", GetParam());
 }
 
 IN_PROC_BROWSER_TEST_P(MediaTest, VideoBearMovPcmS24be) {
-  PlayVideo("bear_pcm_s24be.mov", GetParam());
+  PlayAudio("bear_pcm_s24be.mov", GetParam());
 }
 
 #if BUILDFLAG(USE_PROPRIETARY_CODECS)
@@ -277,16 +298,10 @@ IN_PROC_BROWSER_TEST_P(MediaTest, AudioBearFlac) {
 }
 
 IN_PROC_BROWSER_TEST_P(MediaTest, AudioBearFlacOgg) {
-  PlayVideo("bear-flac.ogg", GetParam());
+  PlayAudio("bear-flac.ogg", GetParam());
 }
 
-// Flaky on Linux. See https://crbug.com/979259
-#if defined(OS_LINUX)
-#define MAYBE_VideoBearWavAlaw DISABLED_VideoBearWavAlaw
-#else
-#define MAYBE_VideoBearWavAlaw VideoBearWavAlaw
-#endif
-IN_PROC_BROWSER_TEST_P(MediaTest, MAYBE_VideoBearWavAlaw) {
+IN_PROC_BROWSER_TEST_P(MediaTest, VideoBearWavAlaw) {
   PlayAudio("bear_alaw.wav", GetParam());
 }
 
@@ -330,10 +345,17 @@ IN_PROC_BROWSER_TEST_P(MediaTest, VideoErrorNoSupportedStreams) {
 // Covers tear-down when navigating away as opposed to browser exiting.
 IN_PROC_BROWSER_TEST_F(MediaTest, Navigate) {
   PlayVideo("bear.webm", false);
-  NavigateToURL(shell(), GURL(url::kAboutBlankURL));
+  EXPECT_TRUE(NavigateToURL(shell(), GURL(url::kAboutBlankURL)));
   EXPECT_FALSE(shell()->web_contents()->IsCrashed());
 }
 
+IN_PROC_BROWSER_TEST_P(MediaTest, AudioOnly_XHE_AAC_MP4) {
+  if (media::IsSupportedAudioType(
+          {media::kCodecAAC, media::AudioCodecProfile::kXHE_AAC})) {
+    PlayAudio("noise-xhe-aac.mp4", GetParam());
+  }
+}
+
 INSTANTIATE_TEST_SUITE_P(File, MediaTest, ::testing::Values(false));
 INSTANTIATE_TEST_SUITE_P(Http, MediaTest, ::testing::Values(true));
 
