@@ -11,7 +11,8 @@
 #include <utility>
 
 #include "base/bind.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/bind_helpers.h"
+#include "base/test/task_environment.h"
 #include "base/time/time.h"
 #include "content/browser/renderer_host/input/synthetic_gesture.h"
 #include "content/browser/renderer_host/input/synthetic_gesture_target.h"
@@ -32,7 +33,7 @@
 #include "content/public/test/test_browser_context.h"
 #include "content/test/test_render_view_host.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/WebKit/public/platform/WebInputEvent.h"
+#include "third_party/blink/public/common/input/web_input_event.h"
 #include "ui/gfx/geometry/point.h"
 #include "ui/gfx/geometry/point_f.h"
 #include "ui/gfx/geometry/vector2d.h"
@@ -53,6 +54,7 @@ const int kPointerAssumedStoppedTimeMs = 43;
 const float kTouchSlopInDips = 7.0f;
 const float kMinScalingSpanInDips = 27.5f;
 const int kTouchPointersLength = 16;
+const int kMouseWheelTickMultiplier = 0;
 
 enum TouchGestureType { TOUCH_SCROLL, TOUCH_DRAG };
 
@@ -65,8 +67,11 @@ WebTouchPoint::State ToWebTouchPointState(
       return WebTouchPoint::kStateMoved;
     case SyntheticPointerActionParams::PointerActionType::RELEASE:
       return WebTouchPoint::kStateReleased;
+    case SyntheticPointerActionParams::PointerActionType::CANCEL:
+      return WebTouchPoint::kStateCancelled;
     case SyntheticPointerActionParams::PointerActionType::IDLE:
       return WebTouchPoint::kStateStationary;
+    case SyntheticPointerActionParams::PointerActionType::LEAVE:
     case SyntheticPointerActionParams::PointerActionType::NOT_INITIALIZED:
       NOTREACHED()
           << "Invalid SyntheticPointerActionParams::PointerActionType.";
@@ -85,6 +90,9 @@ WebInputEvent::Type ToWebMouseEventType(
       return WebInputEvent::kMouseMove;
     case SyntheticPointerActionParams::PointerActionType::RELEASE:
       return WebInputEvent::kMouseUp;
+    case SyntheticPointerActionParams::PointerActionType::LEAVE:
+      return WebInputEvent::kMouseLeave;
+    case SyntheticPointerActionParams::PointerActionType::CANCEL:
     case SyntheticPointerActionParams::PointerActionType::IDLE:
     case SyntheticPointerActionParams::PointerActionType::NOT_INITIALIZED:
       NOTREACHED()
@@ -95,6 +103,22 @@ WebInputEvent::Type ToWebMouseEventType(
   return WebInputEvent::kUndefined;
 }
 
+WebInputEvent::Type WebTouchPointStateToEventType(
+    blink::WebTouchPoint::State state) {
+  switch (state) {
+    case blink::WebTouchPoint::kStateReleased:
+      return WebInputEvent::kTouchEnd;
+    case blink::WebTouchPoint::kStatePressed:
+      return WebInputEvent::kTouchStart;
+    case blink::WebTouchPoint::kStateMoved:
+      return WebInputEvent::kTouchMove;
+    case blink::WebTouchPoint::kStateCancelled:
+      return WebInputEvent::kTouchCancel;
+    default:
+      return WebInputEvent::kUndefined;
+  }
+}
+
 class MockSyntheticGesture : public SyntheticGesture {
  public:
   MockSyntheticGesture(bool* finished, int num_steps)
@@ -150,6 +174,11 @@ class MockSyntheticGestureTarget : public SyntheticGestureTarget {
   }
 
   float GetTouchSlopInDips() const override { return kTouchSlopInDips; }
+  float GetSpanSlopInDips() const override { return 2 * kTouchSlopInDips; }
+
+  int GetMouseWheelMinimumGranularity() const override {
+    return kMouseWheelTickMultiplier;
+  }
 
   float GetMinScalingSpanInDips() const override {
     return kMinScalingSpanInDips;
@@ -158,6 +187,14 @@ class MockSyntheticGestureTarget : public SyntheticGestureTarget {
   bool flush_requested() const { return flush_requested_; }
   void ClearFlushRequest() { flush_requested_ = false; }
 
+  void WaitForTargetAck(SyntheticGestureParams::GestureType type,
+                        SyntheticGestureParams::GestureSourceType source,
+                        base::OnceClosure callback) const override {
+    // Must resolve synchronously since FlushInputUntilComplete will try the
+    // next gesture after this one.
+    std::move(callback).Run();
+  }
+
  private:
   bool flush_requested_;
 
@@ -166,7 +203,9 @@ class MockSyntheticGestureTarget : public SyntheticGestureTarget {
 
 class MockMoveGestureTarget : public MockSyntheticGestureTarget {
  public:
-  MockMoveGestureTarget() : total_abs_move_distance_length_(0) {}
+  MockMoveGestureTarget()
+      : total_abs_move_distance_length_(0),
+        granularity_(ui::ScrollGranularity::kScrollByPixel) {}
   ~MockMoveGestureTarget() override {}
 
   gfx::Vector2dF start_to_end_distance() const {
@@ -176,9 +215,12 @@ class MockMoveGestureTarget : public MockSyntheticGestureTarget {
     return total_abs_move_distance_length_;
   }
 
+  ui::ScrollGranularity granularity() const { return granularity_; }
+
  protected:
   gfx::Vector2dF start_to_end_distance_;
   float total_abs_move_distance_length_;
+  ui::ScrollGranularity granularity_;
 };
 
 class MockScrollMouseTarget : public MockMoveGestureTarget {
@@ -193,6 +235,7 @@ class MockScrollMouseTarget : public MockMoveGestureTarget {
     gfx::Vector2dF delta(mouse_wheel_event.delta_x, mouse_wheel_event.delta_y);
     start_to_end_distance_ += delta;
     total_abs_move_distance_length_ += delta.Length();
+    granularity_ = mouse_wheel_event.delta_units;
   }
 };
 
@@ -208,32 +251,52 @@ class MockMoveTouchTarget : public MockMoveGestureTarget {
 
     if (!started_) {
       ASSERT_EQ(touch_event.GetType(), WebInputEvent::kTouchStart);
-      start_.SetPoint(touch_event.touches[0].PositionInWidget().x,
-                      touch_event.touches[0].PositionInWidget().y);
-      last_touch_point_ = gfx::PointF(start_);
+      start_ = touch_event.touches[0].PositionInWidget();
+      last_touch_point_ = start_;
       started_ = true;
     } else {
       ASSERT_NE(touch_event.GetType(), WebInputEvent::kTouchStart);
       ASSERT_NE(touch_event.GetType(), WebInputEvent::kTouchCancel);
 
-      gfx::PointF touch_point(touch_event.touches[0].PositionInWidget().x,
-                              touch_event.touches[0].PositionInWidget().y);
+      gfx::PointF touch_point(touch_event.touches[0].PositionInWidget());
       gfx::Vector2dF delta = touch_point - last_touch_point_;
       total_abs_move_distance_length_ += delta.Length();
 
       if (touch_event.GetType() == WebInputEvent::kTouchEnd)
-        start_to_end_distance_ = touch_point - gfx::PointF(start_);
+        start_to_end_distance_ = touch_point - start_;
 
       last_touch_point_ = touch_point;
     }
   }
 
  protected:
-  gfx::Point start_;
+  gfx::PointF start_;
   gfx::PointF last_touch_point_;
   bool started_;
 };
 
+class MockFlingGestureTarget : public MockMoveGestureTarget {
+ public:
+  MockFlingGestureTarget() : fling_velocity_x_(0), fling_velocity_y_(0) {}
+  ~MockFlingGestureTarget() override {}
+
+  void DispatchInputEventToPlatform(const WebInputEvent& event) override {
+    if (event.GetType() == WebInputEvent::kGestureFlingStart) {
+      const blink::WebGestureEvent& gesture_event =
+          static_cast<const blink::WebGestureEvent&>(event);
+      fling_velocity_x_ = gesture_event.data.fling_start.velocity_x;
+      fling_velocity_y_ = gesture_event.data.fling_start.velocity_y;
+    }
+  }
+
+  float fling_velocity_x() const { return fling_velocity_x_; }
+  float fling_velocity_y() const { return fling_velocity_y_; }
+
+ private:
+  float fling_velocity_x_;
+  float fling_velocity_y_;
+};
+
 class MockDragMouseTarget : public MockMoveGestureTarget {
  public:
   MockDragMouseTarget() : started_(false) {}
@@ -246,8 +309,7 @@ class MockDragMouseTarget : public MockMoveGestureTarget {
       EXPECT_EQ(mouse_event.button, WebMouseEvent::Button::kLeft);
       EXPECT_EQ(mouse_event.click_count, 1);
       EXPECT_EQ(mouse_event.GetType(), WebInputEvent::kMouseDown);
-      start_.SetPoint(mouse_event.PositionInWidget().x,
-                      mouse_event.PositionInWidget().y);
+      start_ = mouse_event.PositionInWidget();
       last_mouse_point_ = start_;
       started_ = true;
     } else {
@@ -335,10 +397,10 @@ class MockSyntheticTouchscreenPinchTouchTarget
     switch (zoom_direction_) {
       case ZOOM_IN:
         return last_pointer_distance_ /
-               (initial_pointer_distance_ + 2 * GetTouchSlopInDips());
+               (initial_pointer_distance_ + GetSpanSlopInDips());
       case ZOOM_OUT:
         return last_pointer_distance_ /
-               (initial_pointer_distance_ - 2 * GetTouchSlopInDips());
+               (initial_pointer_distance_ - GetSpanSlopInDips());
       case ZOOM_DIRECTION_UNKNOWN:
         return 1.0f;
       default:
@@ -460,16 +522,14 @@ class MockSyntheticTapTouchTarget : public MockSyntheticTapGestureTarget {
       case NOT_STARTED:
         EXPECT_EQ(touch_event.GetType(), WebInputEvent::kTouchStart);
         position_ = gfx::PointF(touch_event.touches[0].PositionInWidget());
-        start_time_ = base::TimeDelta::FromMilliseconds(
-            static_cast<int64_t>(touch_event.TimeStampSeconds() * 1000));
+        start_time_ = touch_event.TimeStamp().since_origin();
         state_ = STARTED;
         break;
       case STARTED:
         EXPECT_EQ(touch_event.GetType(), WebInputEvent::kTouchEnd);
         EXPECT_EQ(position_,
                   gfx::PointF(touch_event.touches[0].PositionInWidget()));
-        stop_time_ = base::TimeDelta::FromMilliseconds(
-            static_cast<int64_t>(touch_event.TimeStampSeconds() * 1000));
+        stop_time_ = touch_event.TimeStamp().since_origin();
         state_ = FINISHED;
         break;
       case FINISHED:
@@ -494,8 +554,7 @@ class MockSyntheticTapMouseTarget : public MockSyntheticTapGestureTarget {
         EXPECT_EQ(mouse_event.button, WebMouseEvent::Button::kLeft);
         EXPECT_EQ(mouse_event.click_count, 1);
         position_ = gfx::PointF(mouse_event.PositionInWidget());
-        start_time_ = base::TimeDelta::FromMilliseconds(
-            static_cast<int64_t>(mouse_event.TimeStampSeconds() * 1000));
+        start_time_ = mouse_event.TimeStamp().since_origin();
         state_ = STARTED;
         break;
       case STARTED:
@@ -503,8 +562,7 @@ class MockSyntheticTapMouseTarget : public MockSyntheticTapGestureTarget {
         EXPECT_EQ(mouse_event.button, WebMouseEvent::Button::kLeft);
         EXPECT_EQ(mouse_event.click_count, 1);
         EXPECT_EQ(position_, gfx::PointF(mouse_event.PositionInWidget()));
-        stop_time_ = base::TimeDelta::FromMilliseconds(
-            static_cast<int64_t>(mouse_event.TimeStampSeconds() * 1000));
+        stop_time_ = mouse_event.TimeStamp().since_origin();
         state_ = FINISHED;
         break;
       case FINISHED:
@@ -516,16 +574,20 @@ class MockSyntheticTapMouseTarget : public MockSyntheticTapGestureTarget {
 
 class MockSyntheticPointerActionTarget : public MockSyntheticGestureTarget {
  public:
-  MockSyntheticPointerActionTarget() : num_actions_dispatched_(0) {}
+  MockSyntheticPointerActionTarget() : num_dispatched_pointer_actions_(0) {}
   ~MockSyntheticPointerActionTarget() override {}
 
   WebInputEvent::Type type() const { return type_; }
-  int num_actions_dispatched() const { return num_actions_dispatched_; }
-  void reset_num_actions_dispatched() { num_actions_dispatched_ = 0; }
+  int num_dispatched_pointer_actions() const {
+    return num_dispatched_pointer_actions_;
+  }
+  void reset_num_dispatched_pointer_actions() {
+    num_dispatched_pointer_actions_ = 0;
+  }
 
  protected:
   WebInputEvent::Type type_;
-  int num_actions_dispatched_;
+  int num_dispatched_pointer_actions_;
 };
 
 class MockSyntheticPointerTouchActionTarget
@@ -539,25 +601,28 @@ class MockSyntheticPointerTouchActionTarget
     const WebTouchEvent& touch_event = static_cast<const WebTouchEvent&>(event);
     type_ = touch_event.GetType();
     for (size_t i = 0; i < WebTouchEvent::kTouchesLengthCap; ++i) {
-      indexes_[i] = touch_event.touches[i].id;
-      positions_[i] = gfx::PointF(touch_event.touches[i].PositionInWidget());
-      states_[i] = touch_event.touches[i].state;
+      if (WebTouchPointStateToEventType(touch_event.touches[i].state) != type_)
+        continue;
+      indexes_[num_dispatched_pointer_actions_] = i;
+      positions_[num_dispatched_pointer_actions_] =
+          gfx::PointF(touch_event.touches[i].PositionInWidget());
+      states_[num_dispatched_pointer_actions_] = touch_event.touches[i].state;
+      num_dispatched_pointer_actions_++;
     }
-    touch_length_ = touch_event.touches_length;
-    num_actions_dispatched_++;
   }
 
   testing::AssertionResult SyntheticTouchActionDispatchedCorrectly(
       const SyntheticPointerActionParams& param,
-      int index) {
+      int index,
+      int touch_index) {
     if (param.pointer_action_type() ==
             SyntheticPointerActionParams::PointerActionType::PRESS ||
         param.pointer_action_type() ==
             SyntheticPointerActionParams::PointerActionType::MOVE) {
-      if (indexes_[index] != param.index()) {
+      if (indexes_[index] != touch_index) {
         return testing::AssertionFailure()
                << "Pointer index at index " << index << " was "
-               << indexes_[index] << ", expected " << param.index() << ".";
+               << indexes_[index] << ", expected " << touch_index << ".";
       }
 
       if (positions_[index] != param.position()) {
@@ -578,17 +643,15 @@ class MockSyntheticPointerTouchActionTarget
   }
 
   testing::AssertionResult SyntheticTouchActionListDispatchedCorrectly(
-      const std::vector<SyntheticPointerActionParams>& params_list) {
-    if (touch_length_ != params_list.size()) {
-      return testing::AssertionFailure() << "Touch point length was "
-                                         << touch_length_ << ", expected "
-                                         << params_list.size() << ".";
-    }
-
+      const std::vector<SyntheticPointerActionParams>& params_list,
+      int start_index,
+      int index_array[]) {
     testing::AssertionResult result = testing::AssertionSuccess();
     for (size_t i = 0; i < params_list.size(); ++i) {
-      result = SyntheticTouchActionDispatchedCorrectly(params_list[i],
-                                                       params_list[i].index());
+      if (params_list[i].pointer_action_type() !=
+          SyntheticPointerActionParams::PointerActionType::IDLE)
+        result = SyntheticTouchActionDispatchedCorrectly(
+            params_list[i], start_index + i, index_array[i]);
       if (result == testing::AssertionFailure())
         return result;
     }
@@ -597,7 +660,6 @@ class MockSyntheticPointerTouchActionTarget
 
  private:
   gfx::PointF positions_[kTouchPointersLength];
-  unsigned touch_length_;
   int indexes_[kTouchPointersLength];
   WebTouchPoint::State states_[kTouchPointersLength];
 };
@@ -615,12 +677,14 @@ class MockSyntheticPointerMouseActionTarget
     position_ = gfx::PointF(mouse_event.PositionInWidget());
     clickCount_ = mouse_event.click_count;
     button_ = mouse_event.button;
-    num_actions_dispatched_++;
+    num_dispatched_pointer_actions_++;
   }
 
   testing::AssertionResult SyntheticMouseActionDispatchedCorrectly(
       const SyntheticPointerActionParams& param,
-      int click_count) {
+      int click_count,
+      SyntheticPointerActionParams::Button button =
+          SyntheticPointerActionParams::Button::NO_BUTTON) {
     if (type_ != ToWebMouseEventType(param.pointer_action_type())) {
       return testing::AssertionFailure()
              << "Pointer type was " << WebInputEvent::GetName(type_)
@@ -634,16 +698,31 @@ class MockSyntheticPointerMouseActionTarget
                                          << click_count << ".";
     }
 
-    if (clickCount_ == 1 && button_ != WebMouseEvent::Button::kLeft) {
-      return testing::AssertionFailure()
-             << "Pointer button was " << (int)button_ << ", expected "
-             << (int)WebMouseEvent::Button::kLeft << ".";
-    }
+    if (button_ != WebMouseEvent::Button::kNoButton) {
+      if (param.pointer_action_type() ==
+              SyntheticPointerActionParams::PointerActionType::PRESS ||
+          param.pointer_action_type() ==
+              SyntheticPointerActionParams::PointerActionType::RELEASE) {
+        if (clickCount_ != 1) {
+          return testing::AssertionFailure() << "Pointer click count was "
+                                             << clickCount_ << ", expected 1.";
+        }
+      }
 
-    if (clickCount_ == 0 && button_ != WebMouseEvent::Button::kNoButton) {
-      return testing::AssertionFailure()
-             << "Pointer button was " << (int)button_ << ", expected "
-             << (int)WebMouseEvent::Button::kNoButton << ".";
+      if (param.pointer_action_type() ==
+          SyntheticPointerActionParams::PointerActionType::MOVE) {
+        if (clickCount_ != 0) {
+          return testing::AssertionFailure() << "Pointer click count was "
+                                             << clickCount_ << ", expected 0.";
+        }
+      }
+
+      if (button_ !=
+          SyntheticPointerActionParams::GetWebMouseEventButton(button)) {
+        return testing::AssertionFailure()
+               << "Pointer button was " << static_cast<int>(button_)
+               << ", expected " << static_cast<int>(button) << ".";
+      }
     }
 
     if ((param.pointer_action_type() ==
@@ -688,14 +767,14 @@ class SyntheticGestureControllerTestBase {
   template<typename MockGestureTarget>
   void CreateControllerAndTarget() {
     target_ = new MockGestureTarget();
-    controller_ = base::MakeUnique<SyntheticGestureController>(
+    controller_ = std::make_unique<SyntheticGestureController>(
         &delegate_, std::unique_ptr<SyntheticGestureTarget>(target_));
   }
 
   void QueueSyntheticGesture(std::unique_ptr<SyntheticGesture> gesture) {
     controller_->QueueSyntheticGesture(
         std::move(gesture),
-        base::Bind(
+        base::BindOnce(
             &SyntheticGestureControllerTestBase::OnSyntheticGestureCompleted,
             base::Unretained(this)));
   }
@@ -703,9 +782,8 @@ class SyntheticGestureControllerTestBase {
   void FlushInputUntilComplete() {
     // Start and stop the timer explicitly here, since the test does not need to
     // wait for begin-frame to start the timer.
-    controller_->dispatch_timer_.Start(FROM_HERE,
-                                       base::TimeDelta::FromSeconds(1),
-                                       base::Bind(&base::DoNothing));
+    controller_->dispatch_timer_.Start(
+        FROM_HERE, base::TimeDelta::FromSeconds(1), base::DoNothing());
     do
       time_ += base::TimeDelta::FromMilliseconds(kFlushInputRateInMs);
     while (controller_->DispatchNextEvent(time_));
@@ -720,9 +798,13 @@ class SyntheticGestureControllerTestBase {
       num_failure_++;
   }
 
+  bool DispatchTimerRunning() const {
+    return controller_->dispatch_timer_.IsRunning();
+  }
+
   base::TimeDelta GetTotalTime() const { return time_ - start_time_; }
 
-  base::test::ScopedTaskEnvironment env_;
+  base::test::TaskEnvironment env_;
   MockSyntheticGestureTarget* target_;
   DummySyntheticGestureControllerDelegate delegate_;
   std::unique_ptr<SyntheticGestureController> controller_;
@@ -1161,8 +1243,74 @@ TEST_F(SyntheticGestureControllerTest, MultiScrollGestureMouseHorizontal) {
   // floating point precision issues with diagonal scrolls.
   EXPECT_FLOAT_EQ(params.distances[0].Length() + params.distances[1].Length(),
                   scroll_target->total_abs_move_distance_length());
-  EXPECT_EQ(params.distances[0] + params.distances[1],
-            scroll_target->start_to_end_distance());
+  EXPECT_FLOAT_EQ((params.distances[0] + params.distances[1]).x(),
+                  scroll_target->start_to_end_distance().x());
+}
+
+TEST_F(SyntheticGestureControllerTest, SingleScrollGestureTouchpadSwipe) {
+  CreateControllerAndTarget<MockFlingGestureTarget>();
+
+  SyntheticSmoothMoveGestureParams params;
+  params.input_type = SyntheticSmoothMoveGestureParams::MOUSE_WHEEL_INPUT;
+  params.start_point.SetPoint(39, 86);
+  params.distances.push_back(gfx::Vector2d(0, -132));
+  params.fling_velocity_x = 800;
+  params.fling_velocity_y = -1000;
+  params.prevent_fling = false;
+
+  std::unique_ptr<SyntheticSmoothMoveGesture> gesture(
+      new SyntheticSmoothMoveGesture(params));
+  QueueSyntheticGesture(std::move(gesture));
+  FlushInputUntilComplete();
+
+  MockFlingGestureTarget* swipe_target =
+      static_cast<MockFlingGestureTarget*>(target_);
+  EXPECT_EQ(1, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(params.fling_velocity_x, swipe_target->fling_velocity_x());
+  EXPECT_EQ(params.fling_velocity_y, swipe_target->fling_velocity_y());
+}
+
+TEST_F(SyntheticGestureControllerTest, SingleScrollGestureMousePreciseScroll) {
+  CreateControllerAndTarget<MockScrollMouseTarget>();
+
+  SyntheticSmoothMoveGestureParams params;
+  params.input_type = SyntheticSmoothMoveGestureParams::MOUSE_WHEEL_INPUT;
+  params.start_point.SetPoint(39, 86);
+  params.distances.push_back(gfx::Vector2d(0, -132));
+  params.granularity = ui::ScrollGranularity::kScrollByPrecisePixel;
+
+  std::unique_ptr<SyntheticSmoothMoveGesture> gesture(
+      new SyntheticSmoothMoveGesture(params));
+  QueueSyntheticGesture(std::move(gesture));
+  FlushInputUntilComplete();
+
+  MockMoveGestureTarget* scroll_target =
+      static_cast<MockMoveGestureTarget*>(target_);
+  EXPECT_EQ(1, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(params.granularity, scroll_target->granularity());
+}
+
+TEST_F(SyntheticGestureControllerTest, SingleScrollGestureMouseScrollByPage) {
+  CreateControllerAndTarget<MockScrollMouseTarget>();
+
+  SyntheticSmoothMoveGestureParams params;
+  params.input_type = SyntheticSmoothMoveGestureParams::MOUSE_WHEEL_INPUT;
+  params.start_point.SetPoint(39, 86);
+  params.distances.push_back(gfx::Vector2d(0, -132));
+  params.granularity = ui::ScrollGranularity::kScrollByPage;
+
+  std::unique_ptr<SyntheticSmoothMoveGesture> gesture(
+      new SyntheticSmoothMoveGesture(params));
+  QueueSyntheticGesture(std::move(gesture));
+  FlushInputUntilComplete();
+
+  MockMoveGestureTarget* scroll_target =
+      static_cast<MockMoveGestureTarget*>(target_);
+  EXPECT_EQ(1, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(params.granularity, scroll_target->granularity());
 }
 
 void CheckIsWithinRangeMulti(float scroll_distance,
@@ -1247,9 +1395,9 @@ TEST_P(SyntheticGestureControllerTestWithParam,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(Single,
-                        SyntheticGestureControllerTestWithParam,
-                        testing::Values(TOUCH_SCROLL, TOUCH_DRAG));
+INSTANTIATE_TEST_SUITE_P(Single,
+                         SyntheticGestureControllerTestWithParam,
+                         testing::Values(TOUCH_SCROLL, TOUCH_DRAG));
 
 TEST_F(SyntheticGestureControllerTest, SingleDragGestureMouseDiagonal) {
   CreateControllerAndTarget<MockDragMouseTarget>();
@@ -1638,9 +1786,9 @@ TEST_F(SyntheticGestureControllerTest, PointerTouchAction) {
   SyntheticPointerActionParams param1 = SyntheticPointerActionParams(
       SyntheticPointerActionParams::PointerActionType::PRESS);
   param0.set_position(gfx::PointF(54, 89));
-  param0.set_index(0);
+  param0.set_pointer_id(0);
   param1.set_position(gfx::PointF(79, 132));
-  param1.set_index(1);
+  param1.set_pointer_id(1);
   param_list.push_back(param0);
   param_list.push_back(param1);
   SyntheticPointerActionListParams params(param_list);
@@ -1652,11 +1800,12 @@ TEST_F(SyntheticGestureControllerTest, PointerTouchAction) {
 
   MockSyntheticPointerTouchActionTarget* pointer_touch_target =
       static_cast<MockSyntheticPointerTouchActionTarget*>(target_);
+  int index_array[2] = {0, 1};
   EXPECT_EQ(1, num_success_);
   EXPECT_EQ(0, num_failure_);
-  EXPECT_EQ(pointer_touch_target->num_actions_dispatched(), 1);
+  EXPECT_EQ(pointer_touch_target->num_dispatched_pointer_actions(), 2);
   EXPECT_TRUE(pointer_touch_target->SyntheticTouchActionListDispatchedCorrectly(
-      param_list));
+      param_list, 0, index_array));
 
   // Second, send a touch release for finger 0, a touch move for finger 1.
   param0.set_pointer_action_type(
@@ -1670,14 +1819,15 @@ TEST_F(SyntheticGestureControllerTest, PointerTouchAction) {
   params.PushPointerActionParamsList(param_list);
   gesture.reset(new SyntheticPointerAction(params));
   QueueSyntheticGesture(std::move(gesture));
-  pointer_touch_target->reset_num_actions_dispatched();
+  pointer_touch_target->reset_num_dispatched_pointer_actions();
   FlushInputUntilComplete();
 
+  index_array[1] = 0;
   EXPECT_EQ(2, num_success_);
   EXPECT_EQ(0, num_failure_);
-  EXPECT_EQ(pointer_touch_target->num_actions_dispatched(), 2);
+  EXPECT_EQ(pointer_touch_target->num_dispatched_pointer_actions(), 4);
   EXPECT_TRUE(pointer_touch_target->SyntheticTouchActionListDispatchedCorrectly(
-      param_list));
+      param_list, 2, index_array));
 
   // Third, send a touch release for finger 1.
   param1.set_pointer_action_type(
@@ -1687,14 +1837,14 @@ TEST_F(SyntheticGestureControllerTest, PointerTouchAction) {
   params.PushPointerActionParamsList(param_list);
   gesture.reset(new SyntheticPointerAction(params));
   QueueSyntheticGesture(std::move(gesture));
-  pointer_touch_target->reset_num_actions_dispatched();
+  pointer_touch_target->reset_num_dispatched_pointer_actions();
   FlushInputUntilComplete();
 
   EXPECT_EQ(3, num_success_);
   EXPECT_EQ(0, num_failure_);
-  EXPECT_EQ(pointer_touch_target->num_actions_dispatched(), 3);
+  EXPECT_EQ(pointer_touch_target->num_dispatched_pointer_actions(), 5);
   EXPECT_TRUE(pointer_touch_target->SyntheticTouchActionListDispatchedCorrectly(
-      param_list));
+      param_list, 4, index_array));
 }
 
 TEST_F(SyntheticGestureControllerTest, PointerMouseAction) {
@@ -1718,7 +1868,7 @@ TEST_F(SyntheticGestureControllerTest, PointerMouseAction) {
       static_cast<MockSyntheticPointerMouseActionTarget*>(target_);
   EXPECT_EQ(1, num_success_);
   EXPECT_EQ(0, num_failure_);
-  EXPECT_EQ(pointer_mouse_target->num_actions_dispatched(), 1);
+  EXPECT_EQ(pointer_mouse_target->num_dispatched_pointer_actions(), 1);
   EXPECT_TRUE(
       pointer_mouse_target->SyntheticMouseActionDispatchedCorrectly(param, 0));
 
@@ -1729,14 +1879,14 @@ TEST_F(SyntheticGestureControllerTest, PointerMouseAction) {
   params.PushPointerActionParams(param);
   gesture.reset(new SyntheticPointerAction(params));
   QueueSyntheticGesture(std::move(gesture));
-  pointer_mouse_target->reset_num_actions_dispatched();
+  pointer_mouse_target->reset_num_dispatched_pointer_actions();
   FlushInputUntilComplete();
 
   EXPECT_EQ(2, num_success_);
   EXPECT_EQ(0, num_failure_);
-  EXPECT_EQ(pointer_mouse_target->num_actions_dispatched(), 2);
-  EXPECT_TRUE(
-      pointer_mouse_target->SyntheticMouseActionDispatchedCorrectly(param, 1));
+  EXPECT_EQ(pointer_mouse_target->num_dispatched_pointer_actions(), 2);
+  EXPECT_TRUE(pointer_mouse_target->SyntheticMouseActionDispatchedCorrectly(
+      param, 1, SyntheticPointerActionParams::Button::LEFT));
 
   // Third, send a mouse move.
   param.set_pointer_action_type(
@@ -1745,14 +1895,14 @@ TEST_F(SyntheticGestureControllerTest, PointerMouseAction) {
   params.PushPointerActionParams(param);
   gesture.reset(new SyntheticPointerAction(params));
   QueueSyntheticGesture(std::move(gesture));
-  pointer_mouse_target->reset_num_actions_dispatched();
+  pointer_mouse_target->reset_num_dispatched_pointer_actions();
   FlushInputUntilComplete();
 
   EXPECT_EQ(3, num_success_);
   EXPECT_EQ(0, num_failure_);
-  EXPECT_EQ(pointer_mouse_target->num_actions_dispatched(), 3);
-  EXPECT_TRUE(
-      pointer_mouse_target->SyntheticMouseActionDispatchedCorrectly(param, 1));
+  EXPECT_EQ(pointer_mouse_target->num_dispatched_pointer_actions(), 3);
+  EXPECT_TRUE(pointer_mouse_target->SyntheticMouseActionDispatchedCorrectly(
+      param, 0, SyntheticPointerActionParams::Button::LEFT));
 
   // Fourth, send a mouse release.
   param.set_pointer_action_type(
@@ -1760,14 +1910,176 @@ TEST_F(SyntheticGestureControllerTest, PointerMouseAction) {
   params.PushPointerActionParams(param);
   gesture.reset(new SyntheticPointerAction(params));
   QueueSyntheticGesture(std::move(gesture));
-  pointer_mouse_target->reset_num_actions_dispatched();
+  pointer_mouse_target->reset_num_dispatched_pointer_actions();
   FlushInputUntilComplete();
 
   EXPECT_EQ(4, num_success_);
   EXPECT_EQ(0, num_failure_);
-  EXPECT_EQ(pointer_mouse_target->num_actions_dispatched(), 4);
+  EXPECT_EQ(pointer_mouse_target->num_dispatched_pointer_actions(), 4);
+  EXPECT_TRUE(pointer_mouse_target->SyntheticMouseActionDispatchedCorrectly(
+      param, 1, SyntheticPointerActionParams::Button::LEFT));
+}
+
+TEST_F(SyntheticGestureControllerTest, PointerPenAction) {
+  CreateControllerAndTarget<MockSyntheticPointerMouseActionTarget>();
+
+  // First, send a pen move.
+  SyntheticPointerActionListParams::ParamList param_list;
+  SyntheticPointerActionParams param = SyntheticPointerActionParams(
+      SyntheticPointerActionParams::PointerActionType::MOVE);
+
+  param.set_position(gfx::PointF(54, 89));
+  SyntheticPointerActionListParams params;
+  params.PushPointerActionParams(param);
+  params.gesture_source_type = SyntheticGestureParams::PEN_INPUT;
+  std::unique_ptr<SyntheticPointerAction> gesture(
+      new SyntheticPointerAction(params));
+  QueueSyntheticGesture(std::move(gesture));
+  FlushInputUntilComplete();
+
+  MockSyntheticPointerMouseActionTarget* pointer_pen_target =
+      static_cast<MockSyntheticPointerMouseActionTarget*>(target_);
+  EXPECT_EQ(1, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(pointer_pen_target->num_dispatched_pointer_actions(), 1);
   EXPECT_TRUE(
-      pointer_mouse_target->SyntheticMouseActionDispatchedCorrectly(param, 1));
+      pointer_pen_target->SyntheticMouseActionDispatchedCorrectly(param, 0));
+
+  // Second, send a pen press.
+  param.set_pointer_action_type(
+      SyntheticPointerActionParams::PointerActionType::PRESS);
+  param.set_position(gfx::PointF(183, 239));
+  params.PushPointerActionParams(param);
+  gesture.reset(new SyntheticPointerAction(params));
+  QueueSyntheticGesture(std::move(gesture));
+  pointer_pen_target->reset_num_dispatched_pointer_actions();
+  FlushInputUntilComplete();
+
+  EXPECT_EQ(2, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(pointer_pen_target->num_dispatched_pointer_actions(), 2);
+  EXPECT_TRUE(pointer_pen_target->SyntheticMouseActionDispatchedCorrectly(
+      param, 1, SyntheticPointerActionParams::Button::LEFT));
+
+  // Third, send a pen move.
+  param.set_pointer_action_type(
+      SyntheticPointerActionParams::PointerActionType::MOVE);
+  param.set_position(gfx::PointF(254, 279));
+  params.PushPointerActionParams(param);
+  gesture.reset(new SyntheticPointerAction(params));
+  QueueSyntheticGesture(std::move(gesture));
+  pointer_pen_target->reset_num_dispatched_pointer_actions();
+  FlushInputUntilComplete();
+
+  EXPECT_EQ(3, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(pointer_pen_target->num_dispatched_pointer_actions(), 3);
+  EXPECT_TRUE(pointer_pen_target->SyntheticMouseActionDispatchedCorrectly(
+      param, 0, SyntheticPointerActionParams::Button::LEFT));
+
+  // Fourth, send a pen release.
+  param.set_pointer_action_type(
+      SyntheticPointerActionParams::PointerActionType::RELEASE);
+  params.PushPointerActionParams(param);
+  gesture.reset(new SyntheticPointerAction(params));
+  QueueSyntheticGesture(std::move(gesture));
+  pointer_pen_target->reset_num_dispatched_pointer_actions();
+  FlushInputUntilComplete();
+
+  EXPECT_EQ(4, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(pointer_pen_target->num_dispatched_pointer_actions(), 4);
+  EXPECT_TRUE(pointer_pen_target->SyntheticMouseActionDispatchedCorrectly(
+      param, 1, SyntheticPointerActionParams::Button::LEFT));
+
+  // Fifth, send a pen leave.
+  param.set_pointer_action_type(
+      SyntheticPointerActionParams::PointerActionType::LEAVE);
+  params.PushPointerActionParams(param);
+  gesture.reset(new SyntheticPointerAction(params));
+  QueueSyntheticGesture(std::move(gesture));
+  pointer_pen_target->reset_num_dispatched_pointer_actions();
+  FlushInputUntilComplete();
+
+  EXPECT_EQ(5, num_success_);
+  EXPECT_EQ(0, num_failure_);
+  EXPECT_EQ(pointer_pen_target->num_dispatched_pointer_actions(), 5);
+  EXPECT_TRUE(
+      pointer_pen_target->SyntheticMouseActionDispatchedCorrectly(param, 0));
+}
+
+class MockSyntheticGestureTargetManualAck : public MockSyntheticGestureTarget {
+ public:
+  void WaitForTargetAck(SyntheticGestureParams::GestureType type,
+                        SyntheticGestureParams::GestureSourceType source,
+                        base::OnceClosure callback) const override {
+    if (manually_ack_)
+      target_ack_ = std::move(callback);
+    else
+      std::move(callback).Run();
+  }
+  bool HasOutstandingAck() const { return !target_ack_.is_null(); }
+  void InvokeAck() {
+    DCHECK(HasOutstandingAck());
+    std::move(target_ack_).Run();
+  }
+  void SetManuallyAck(bool manually_ack) { manually_ack_ = manually_ack; }
+
+ private:
+  mutable base::OnceClosure target_ack_;
+  bool manually_ack_ = true;
+};
+
+// Ensure the first time a gesture is queued, we wait for a renderer ACK before
+// starting the gesture. Following gestures should start immediately. This test
+// the renderer_known_to_be_initialized_ bit in the controller.
+TEST_F(SyntheticGestureControllerTest, WaitForRendererInitialization) {
+  CreateControllerAndTarget<MockSyntheticGestureTargetManualAck>();
+
+  auto* target = static_cast<MockSyntheticGestureTargetManualAck*>(target_);
+
+  EXPECT_FALSE(target->HasOutstandingAck());
+
+  SyntheticTapGestureParams params;
+  params.gesture_source_type = SyntheticGestureParams::TOUCH_INPUT;
+  params.duration_ms = 123;
+  params.position.SetPoint(87, -124);
+
+  // Queue the first gesture.
+  {
+    auto gesture = std::make_unique<SyntheticTapGesture>(params);
+    QueueSyntheticGesture(std::move(gesture));
+
+    // We should have received a WaitForTargetAck and the dispatch timer won't
+    // start until that's ACK'd.
+    EXPECT_TRUE(target->HasOutstandingAck());
+    EXPECT_FALSE(DispatchTimerRunning());
+
+    target->InvokeAck();
+
+    // The timer should now be running.
+    EXPECT_FALSE(target->HasOutstandingAck());
+    EXPECT_TRUE(DispatchTimerRunning());
+  }
+
+  // Finish the gesture.
+  {
+    target->SetManuallyAck(false);
+    FlushInputUntilComplete();
+    target->SetManuallyAck(true);
+    EXPECT_FALSE(DispatchTimerRunning());
+  }
+
+  // Queue the second gesture.
+  {
+    auto gesture = std::make_unique<SyntheticTapGesture>(params);
+    QueueSyntheticGesture(std::move(gesture));
+
+    // This time, because we've already sent a gesuture to the renderer,
+    // there's no need to wait for an ACK before starting the dispatch timer.
+    EXPECT_FALSE(target->HasOutstandingAck());
+    EXPECT_TRUE(DispatchTimerRunning());
+  }
 }
 
 }  // namespace content
