@@ -35,12 +35,15 @@
 
 #include <stdint.h>
 
-#include "base/mac/sdk_forward_declarations.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_util.h"
-#include "third_party/WebKit/public/platform/WebInputEvent.h"
+#include "base/time/time.h"
+#include "third_party/blink/public/common/input/web_input_event.h"
 #include "ui/base/cocoa/cocoa_base_utils.h"
+#include "ui/events/base_event_utils.h"
 #include "ui/events/blink/blink_event_util.h"
 #import "ui/events/cocoa/cocoa_event_utils.h"
+#include "ui/events/event_utils.h"
 #include "ui/events/keycodes/keyboard_code_conversion.h"
 #include "ui/events/keycodes/keyboard_code_conversion_mac.h"
 
@@ -106,6 +109,10 @@ int ModifiersFromEvent(NSEvent* event) {
     modifiers |= blink::WebInputEvent::kRightButtonDown;
   if (pressed_buttons & (1 << 2))
     modifiers |= blink::WebInputEvent::kMiddleButtonDown;
+  if (pressed_buttons & (1 << 3))
+    modifiers |= blink::WebInputEvent::kBackButtonDown;
+  if (pressed_buttons & (1 << 4))
+    modifiers |= blink::WebInputEvent::kForwardButtonDown;
 
   return modifiers;
 }
@@ -195,9 +202,54 @@ ui::DomKey DomKeyFromEvent(NSEvent* event) {
   return ui::DomKey::UNIDENTIFIED;
 }
 
+blink::WebMouseEvent::Button ButtonFromPressedMouseButtons() {
+  NSUInteger pressed_buttons = [NSEvent pressedMouseButtons];
+
+  if (pressed_buttons & (1 << 0))
+    return blink::WebMouseEvent::Button::kLeft;
+  if (pressed_buttons & (1 << 1))
+    return blink::WebMouseEvent::Button::kRight;
+  if (pressed_buttons & (1 << 2))
+    return blink::WebMouseEvent::Button::kMiddle;
+  if (pressed_buttons & (1 << 3))
+    return blink::WebMouseEvent::Button::kBack;
+  if (pressed_buttons & (1 << 4))
+    return blink::WebMouseEvent::Button::kForward;
+  return blink::WebMouseEvent::Button::kNoButton;
+}
+blink::WebMouseEvent::Button ButtonFromButtonNumber(NSEvent* event) {
+  NSUInteger button_number = [event buttonNumber];
+
+  if (button_number == 1)
+    return blink::WebMouseEvent::Button::kRight;
+  if (button_number == 2)
+    return blink::WebMouseEvent::Button::kMiddle;
+  if (button_number == 3)
+    return blink::WebMouseEvent::Button::kBack;
+  if (button_number == 4)
+    return blink::WebMouseEvent::Button::kForward;
+  return blink::WebMouseEvent::Button::kNoButton;
+}
+
 }  // namespace
 
-blink::WebKeyboardEvent WebKeyboardEventBuilder::Build(NSEvent* event) {
+blink::WebKeyboardEvent WebKeyboardEventBuilder::Build(NSEvent* event,
+                                                       bool record_debug_uma) {
+  ui::ComputeEventLatencyOS(event);
+  base::TimeTicks now = ui::EventTimeForNow();
+  base::TimeTicks hardware_timestamp =
+      ui::EventTimeStampFromSeconds([event timestamp]);
+  if (record_debug_uma) {
+    if (ui::EventTypeFromNative(event) == ui::ET_KEY_PRESSED) {
+      UMA_HISTOGRAM_CUSTOM_TIMES(
+          now > hardware_timestamp
+              ? "Event.Latency.OS_NO_VALIDATION.POSITIVE.KEY_PRESSED"
+              : "Event.Latency.OS_NO_VALIDATION.NEGATIVE.KEY_PRESSED",
+          (now - hardware_timestamp).magnitude(),
+          base::TimeDelta::FromMilliseconds(1),
+          base::TimeDelta::FromSeconds(60), 50);
+    }
+  }
   ui::DomCode dom_code = ui::DomCodeFromNSEvent(event);
   int modifiers =
       ModifiersFromEvent(event) | ui::DomCodeToWebInputEventModifiers(dom_code);
@@ -205,10 +257,10 @@ blink::WebKeyboardEvent WebKeyboardEventBuilder::Build(NSEvent* event) {
   if (([event type] != NSFlagsChanged) && [event isARepeat])
     modifiers |= blink::WebInputEvent::kIsAutoRepeat;
 
-  blink::WebKeyboardEvent result(ui::IsKeyUpEvent(event)
-                                     ? blink::WebInputEvent::kKeyUp
-                                     : blink::WebInputEvent::kRawKeyDown,
-                                 modifiers, [event timestamp]);
+  blink::WebKeyboardEvent result(
+      ui::IsKeyUpEvent(event) ? blink::WebInputEvent::kKeyUp
+                              : blink::WebInputEvent::kRawKeyDown,
+      modifiers, ui::EventTimeStampFromSeconds([event timestamp]));
   result.windows_key_code =
       ui::LocatedToNonLocatedKeyboardCode(ui::KeyboardCodeFromNSEvent(event));
   result.native_key_code = [event keyCode];
@@ -253,6 +305,19 @@ blink::WebMouseEvent WebMouseEventBuilder::Build(
     NSEvent* event,
     NSView* view,
     blink::WebPointerProperties::PointerType pointerType) {
+  ui::ComputeEventLatencyOS(event);
+  base::TimeTicks now = ui::EventTimeForNow();
+  base::TimeTicks hardware_timestamp =
+      ui::EventTimeStampFromSeconds([event timestamp]);
+  if (ui::EventTypeFromNative(event) == ui::ET_MOUSE_PRESSED) {
+    UMA_HISTOGRAM_CUSTOM_TIMES(
+        now > hardware_timestamp
+            ? "Event.Latency.OS_NO_VALIDATION.POSITIVE.MOUSE_PRESSED"
+            : "Event.Latency.OS_NO_VALIDATION.NEGATIVE.MOUSE_PRESSED",
+        (now - hardware_timestamp).magnitude(),
+        base::TimeDelta::FromMilliseconds(1), base::TimeDelta::FromSeconds(60),
+        50);
+  }
   blink::WebInputEvent::Type event_type =
       blink::WebInputEvent::Type::kUndefined;
   int click_count = 0;
@@ -271,7 +336,7 @@ blink::WebMouseEvent WebMouseEventBuilder::Build(
     case NSOtherMouseDown:
       event_type = blink::WebInputEvent::kMouseDown;
       click_count = [event clickCount];
-      button = blink::WebMouseEvent::Button::kMiddle;
+      button = ButtonFromButtonNumber(event);
       break;
     case NSRightMouseDown:
       event_type = blink::WebInputEvent::kMouseDown;
@@ -286,7 +351,7 @@ blink::WebMouseEvent WebMouseEventBuilder::Build(
     case NSOtherMouseUp:
       event_type = blink::WebInputEvent::kMouseUp;
       click_count = [event clickCount];
-      button = blink::WebMouseEvent::Button::kMiddle;
+      button = ButtonFromButtonNumber(event);
       break;
     case NSRightMouseUp:
       event_type = blink::WebInputEvent::kMouseUp;
@@ -296,6 +361,7 @@ blink::WebMouseEvent WebMouseEventBuilder::Build(
     case NSMouseMoved:
     case NSMouseEntered:
       event_type = blink::WebInputEvent::kMouseMove;
+      button = ButtonFromPressedMouseButtons();
       break;
     case NSLeftMouseDragged:
       event_type = blink::WebInputEvent::kMouseMove;
@@ -317,7 +383,8 @@ blink::WebMouseEvent WebMouseEventBuilder::Build(
   // NSMouseExited and NSMouseEntered events don't have deviceID.
   // Therefore pen exit and enter events can't get correct id.
   blink::WebMouseEvent result(event_type, ModifiersFromEvent(event),
-                              [event timestamp], 0);
+                              ui::EventTimeStampFromSeconds([event timestamp]),
+                              0);
   result.click_count = click_count;
   result.button = button;
   SetWebEventLocationFromEventInView(&result, event, view);
@@ -359,9 +426,20 @@ blink::WebMouseEvent WebMouseEventBuilder::Build(
 blink::WebMouseWheelEvent WebMouseWheelEventBuilder::Build(
     NSEvent* event,
     NSView* view) {
-  blink::WebMouseWheelEvent result(blink::WebInputEvent::kMouseWheel,
-                                   ModifiersFromEvent(event),
-                                   [event timestamp]);
+  ui::ComputeEventLatencyOS(event);
+  base::TimeTicks now = ui::EventTimeForNow();
+  base::TimeTicks hardware_timestamp =
+      ui::EventTimeStampFromSeconds([event timestamp]);
+  UMA_HISTOGRAM_CUSTOM_TIMES(
+      now > hardware_timestamp
+          ? "Event.Latency.OS_NO_VALIDATION.POSITIVE.MOUSE_WHEEL"
+          : "Event.Latency.OS_NO_VALIDATION.NEGATIVE.MOUSE_WHEEL",
+      (now - hardware_timestamp).magnitude(),
+      base::TimeDelta::FromMilliseconds(1), base::TimeDelta::FromSeconds(60),
+      50);
+  blink::WebMouseWheelEvent result(
+      blink::WebInputEvent::kMouseWheel, ModifiersFromEvent(event),
+      ui::EventTimeStampFromSeconds([event timestamp]));
   result.button = blink::WebMouseEvent::Button::kNoButton;
 
   SetWebEventLocationFromEventInView(&result, event, view);
@@ -480,13 +558,13 @@ blink::WebMouseWheelEvent WebMouseWheelEventBuilder::Build(
   // from data from any other continuous device.
 
   if (CGEventGetIntegerValueField(cg_event, kCGScrollWheelEventIsContinuous)) {
+    result.delta_units = ui::ScrollGranularity::kScrollByPrecisePixel;
     result.delta_x = CGEventGetIntegerValueField(
         cg_event, kCGScrollWheelEventPointDeltaAxis2);
     result.delta_y = CGEventGetIntegerValueField(
         cg_event, kCGScrollWheelEventPointDeltaAxis1);
     result.wheel_ticks_x = result.delta_x / ui::kScrollbarPixelsPerCocoaTick;
     result.wheel_ticks_y = result.delta_y / ui::kScrollbarPixelsPerCocoaTick;
-    result.has_precise_scrolling_deltas = true;
   } else {
     result.delta_x = [event deltaX] * ui::kScrollbarPixelsPerCocoaTick;
     result.delta_y = [event deltaY] * ui::kScrollbarPixelsPerCocoaTick;
@@ -510,19 +588,22 @@ blink::WebGestureEvent WebGestureEventBuilder::Build(NSEvent* event,
   blink::WebMouseEvent temp;
 
   SetWebEventLocationFromEventInView(&temp, event, view);
-  result.x = temp.PositionInWidget().x;
-  result.y = temp.PositionInWidget().y;
-  result.global_x = temp.PositionInScreen().x;
-  result.global_y = temp.PositionInScreen().y;
+  result.SetPositionInWidget(temp.PositionInWidget());
+  result.SetPositionInScreen(temp.PositionInScreen());
 
   result.SetModifiers(ModifiersFromEvent(event));
-  result.SetTimeStampSeconds([event timestamp]);
+  result.SetTimeStamp(ui::EventTimeStampFromSeconds([event timestamp]));
+
+  result.SetSourceDevice(blink::WebGestureDevice::kTouchpad);
 
-  result.source_device = blink::kWebGestureDeviceTouchpad;
   switch ([event type]) {
     case NSEventTypeMagnify:
+      // We don't need to set the type based on |[event phase]| as the caller
+      // must set the begin and end types in order to support older Mac
+      // versions.
       result.SetType(blink::WebInputEvent::kGesturePinchUpdate);
       result.data.pinch_update.scale = [event magnification] + 1.0;
+      result.SetNeedsWheelEvent(true);
       break;
     case NSEventTypeSmartMagnify:
       // Map the Cocoa "double-tap with two fingers" zoom gesture to regular
@@ -531,6 +612,7 @@ blink::WebGestureEvent WebGestureEventBuilder::Build(NSEvent* event,
       // because the gesture type already encodes that information.
       result.SetType(blink::WebInputEvent::kGestureDoubleTap);
       result.data.tap.tap_count = 1;
+      result.SetNeedsWheelEvent(true);
       break;
     case NSEventTypeBeginGesture:
     case NSEventTypeEndGesture:
@@ -538,6 +620,13 @@ blink::WebGestureEvent WebGestureEventBuilder::Build(NSEvent* event,
       // and end NSEvents come in. Leave them undefined. The caller will need
       // to specify them when the gesture is differentiated.
       break;
+    case NSEventTypeScrollWheel:
+      // When building against the 10.11 SDK or later, and running on macOS
+      // 10.11+, Cocoa no longer sends separate Begin/End gestures for scroll
+      // events. However, it's convenient to use the same path as the older
+      // OSes, to avoid logic duplication. We just need to support building a
+      // dummy WebGestureEvent.
+      break;
     default:
       NOTIMPLEMENTED();
   }
@@ -545,4 +634,85 @@ blink::WebGestureEvent WebGestureEventBuilder::Build(NSEvent* event,
   return result;
 }
 
+// WebTouchEvent --------------------------------------------------------------
+
+blink::WebTouchEvent WebTouchEventBuilder::Build(NSEvent* event, NSView* view) {
+  blink::WebInputEvent::Type event_type =
+      blink::WebInputEvent::Type::kUndefined;
+  NSEventType type = [event type];
+  blink::WebTouchPoint::State state = blink::WebTouchPoint::kStateUndefined;
+  switch (type) {
+    case NSLeftMouseDown:
+      event_type = blink::WebInputEvent::kTouchStart;
+      state = blink::WebTouchPoint::kStatePressed;
+      break;
+    case NSLeftMouseUp:
+      event_type = blink::WebInputEvent::kTouchEnd;
+      state = blink::WebTouchPoint::kStateReleased;
+      break;
+    case NSLeftMouseDragged:
+    case NSRightMouseDragged:
+    case NSOtherMouseDragged:
+    case NSMouseMoved:
+    case NSRightMouseDown:
+    case NSOtherMouseDown:
+    case NSRightMouseUp:
+    case NSOtherMouseUp:
+      event_type = blink::WebInputEvent::kTouchMove;
+      state = blink::WebTouchPoint::kStateMoved;
+      break;
+    default:
+      NOTREACHED() << "Invalid types for touch events." << type;
+  }
+
+  blink::WebTouchEvent result(event_type, ModifiersFromEvent(event),
+                              ui::EventTimeStampFromSeconds([event timestamp]));
+  if (ui::EventTypeFromNative(event) == ui::ET_TOUCH_PRESSED) {
+    base::TimeTicks now = ui::EventTimeForNow();
+    base::TimeTicks hardware_timestamp =
+        ui::EventTimeStampFromSeconds([event timestamp]);
+    UMA_HISTOGRAM_CUSTOM_TIMES(
+        now > hardware_timestamp
+            ? "Event.Latency.OS.NO_VALIDATION.POSITIVE.TOUCH_PRESSED"
+            : "Event.Latency.OS.NO_VALIDATION.NEGATIVE.TOUCH_PRESSED",
+        (now - hardware_timestamp).magnitude(),
+        base::TimeDelta::FromMilliseconds(1), base::TimeDelta::FromSeconds(60),
+        50);
+  }
+  ui::ComputeEventLatencyOS(event);
+  result.hovering = event_type == blink::WebInputEvent::kTouchEnd;
+  result.unique_touch_event_id = ui::GetNextTouchEventId();
+  result.touches_length = 1;
+
+  // Use a temporary WebMouseEvent to get the location.
+  blink::WebMouseEvent temp;
+  SetWebEventLocationFromEventInView(&temp, event, view);
+  result.touches[0].SetPositionInWidget(temp.PositionInWidget());
+  result.touches[0].SetPositionInScreen(temp.PositionInScreen());
+  result.touches[0].movement_x = temp.movement_x;
+  result.touches[0].movement_y = temp.movement_y;
+
+  result.touches[0].state = state;
+  result.touches[0].pointer_type =
+      blink::WebPointerProperties::PointerType::kPen;
+  result.touches[0].id = [event pointingDeviceID];
+  result.touches[0].force = [event pressure];
+  NSPoint tilt = [event tilt];
+  result.touches[0].tilt_x = lround(tilt.x * 90);
+  result.touches[0].tilt_y = lround(tilt.y * 90);
+  result.touches[0].tangential_pressure = [event tangentialPressure];
+  // NSEvent spec doesn't specify the range of rotation, we make sure that
+  // this value is in the range of [0,359].
+  int twist = (int)[event rotation];
+  twist = twist % 360;
+  if (twist < 0)
+    twist += 360;
+  result.touches[0].twist = twist;
+  float rotation_angle = twist % 180;
+  if (rotation_angle > 90)
+    rotation_angle = 180.f - rotation_angle;
+  result.touches[0].rotation_angle = rotation_angle;
+  return result;
+}
+
 }  // namespace content
